//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "UMain.h"
#include "UAbout.h"
#include "UError.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "DisAsmView"
#pragma link "CodeBPList"
#pragma link "Debugger"
#pragma link "ExportsList"
#pragma link "ExecBreakpoint"
#pragma link "DataBPList"
#pragma link "DumpView"
#pragma link "StackView"
#pragma resource "*.dfm"
TFMain *FMain;
//---------------------------------------------------------------------------
__fastcall TFMain::TFMain(TComponent* Owner)
      : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TFMain::FileOpenAccept(TObject *Sender)
{
      Debugger->StartDebugger(FileOpen->Dialog->FileName, "");
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugRunUpdate(TObject *Sender)
{
      aDebugRun->Enabled = Debugger->CanRun();      
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugPauseUpdate(TObject *Sender)
{
      aDebugPause->Enabled = Debugger->CanPause();      
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugStepOverUpdate(TObject *Sender)
{
      aDebugStepOver->Enabled = Debugger->CanStep();
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugStepIntoUpdate(TObject *Sender)
{
      aDebugStepInto->Enabled = Debugger->CanStep();      
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugStopUpdate(TObject *Sender)
{
      aDebugStop->Enabled = Debugger->DebuggerState != dsStopped;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugRunExecute(TObject *Sender)
{
      Debugger->Run();      
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugPauseExecute(TObject *Sender)
{
      Debugger->Pause();      
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugStepOverExecute(TObject *Sender)
{
      Debugger->StepOver();      
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugStepIntoExecute(TObject *Sender)
{
      Debugger->StepInto();      
}
//---------------------------------------------------------------------------

void __fastcall TFMain::aDebugStopExecute(TObject *Sender)
{
      Debugger->Terminate();      
}
//---------------------------------------------------------------------------

void TFMain::ShowCPUState(DWORD dwThreadId)
{
      TContext Context;
      if(!Debugger->GetContext(dwThreadId,CONTEXT_FULL,Context))
      {
            MessageDlg("Unable to get context", mtError, TMsgDlgButtons() << mbOK, 0);
            return;
      };
      edEAX->Text = IntToHex((int)Context.Eax,8);
	edEBX->Text = IntToHex((int)Context.Ebx,8);
	edECX->Text = IntToHex((int)Context.Ecx,8);
	edEDX->Text = IntToHex((int)Context.Edx,8);
	edESI->Text = IntToHex((int)Context.Esi,8);
	edEDI->Text = IntToHex((int)Context.Edi,8);
	edESP->Text = IntToHex((int)Context.Esp,8);
	edEBP->Text = IntToHex((int)Context.Ebp,8);
	edEFLAGS->Text = IntToHex((int)Context.EFlags,8);
	edEIP->Text = IntToHex((int)Context.Eip,8);
}

void __fastcall TFMain::DebuggerChangeDebuggerState(TObject *Sender,
      DWORD dwThreadId, TDebuggerState NewState)
{
      StatusBar1->Panels->Items[2]->Text = "";
      StatusBar1->Panels->Items[0]->Text =
            NewState == dsStopped ? "Stopped" :
            NewState == dsRunning ? "Running" :
            NewState == dsPaused ? "Paused" :
            NewState == dsStepOver ? "Step Over" :
            NewState == dsStepInto ? "Step Into" : "";
      if(NewState == dsPaused || NewState == dsStepInto || NewState == dsStepOver)
      {
            ShowCPUState(dwThreadId);
            DWORD Eip = Debugger->GetEIP(dwThreadId);
            DisAsmView->ShowMemory(Eip);
            StackView->ShowStackMemory();

            TModule *Module = Debugger->DbgProcess->Modules->GetModuleByAddress(Eip);
            AnsiString ModuleName = Module != NULL ? Module->BaseName : (AnsiString)"";
            StatusBar1->Panels->Items[1]->Text = "Module: " + ModuleName;
      }

      if(NewState == dsRunning)
      {
            DWORD Eip = Debugger->GetEIP(dwThreadId);
            int LineIndex = DisAsmView->GetLineByAddress(Eip);
            DisAsmView->InvalidateGutterLine(LineIndex);
       	DisAsmView->InvalidateLine(LineIndex);
            StatusBar1->Panels->Items[1]->Text = "Module: ";
      }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerCreateProcessDbgEvent(TObject *Sender,
      DWORD dwThreadId, CREATE_PROCESS_DEBUG_INFO CreateProcessInfo)
{
      DisAsmView->ProcessID = Debugger->DbgProcess->dwProcessID;
      DumpView->ProcessID = Debugger->DbgProcess->dwProcessID;
      StackView->ProcessID = Debugger->DbgProcess->dwProcessID;

      DumpView->ShowMemory((DWORD)CreateProcessInfo.lpBaseOfImage);
      DisAsmView->SelEnd = DisAsmView->SelBegin;

      TExports* Exports = ExportsList->AddExports(Debugger->DbgProcess->ModulePath);
      if(Exports)
            Exports->CorrectAddresses((DWORD)CreateProcessInfo.lpBaseOfImage);
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerExitProcessDbgEvent(TObject *Sender,
      DWORD dwThreadId, EXIT_PROCESS_DEBUG_INFO ExitProcess)
{
      Debugger->CloseDebugger();
      ExportsList->Clear();
      DisAsmView->ProcessID = 0;
      DisAsmView->Buffer->Clear();
      DumpView->ProcessID = 0;
      DumpView->ShowMemory(0);
      StackView->ProcessID = 0;
      StackView->ShowMemory(0);

      edEAX->Text = "";
	edEBX->Text = "";
	edECX->Text = "";
	edEDX->Text = "";
	edESI->Text = "";
	edEDI->Text = "";
	edESP->Text = "";
	edEBP->Text = "";
	edEFLAGS->Text = "";
	edEIP->Text = "";

      StatusBar1->Panels->Items[0]->Text = "Stopped";
      StatusBar1->Panels->Items[1]->Text = "Module:";
      StatusBar1->Panels->Items[2]->Text = "";
      StatusBar1->Panels->Items[3]->Text = "";
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerCodeBPDetected(TObject *Sender,
      DWORD dwThreadId, TCodeBP *CodeBP)
{
      StatusBar1->Panels->Items[2]->Text =
            "User Breakpoint("+
      	(CodeBP->Function == "" ? IntToHex((int)CodeBP->Address,8) :
            	" '"+CodeBP->Function+"' ") +")";
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerCreateThreadDbgEvent(TObject *Sender,
      DWORD dwThreadId, CREATE_THREAD_DEBUG_INFO CreateThread, bool Pause)
{
      StatusBar1->Panels->Items[2]->Text =
            "New Thread (Start Address - "+
            IntToHex((int)CreateThread.lpStartAddress,8) + ")";
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerDataBPDetected(TObject *Sender,
      DWORD dwThreadId, TDataBP *DataBP)
{
      if(DataBP == NULL) return;
      StatusBar1->Panels->Items[2]->Text =
            "Data Breakpoint (Address: "+
            IntToHex((int)DataBP->Address,8) + ")";
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerDebugStringDbgEvent(TObject *Sender,
      DWORD dwThreadId, AnsiString DebugString,
      OUTPUT_DEBUG_STRING_INFO DebugStringInfo, bool Pause)
{
      StatusBar1->Panels->Items[2]->Text = "Debug string: " + DebugString;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerExceptionDbgEvent(TObject *Sender,
      DWORD dwThreadId, EXCEPTION_DEBUG_INFO Exception)
{
      AnsiString S;
      switch(Exception.ExceptionRecord.ExceptionCode)
      {
      case EXCEPTION_SINGLE_STEP:
            return;
      case EXCEPTION_ACCESS_VIOLATION:
            S = "ACCESS VIOLATION";
      break;
      case EXCEPTION_BREAKPOINT:
            S = "EXCEPTION_BREAKPOINT";
      break;
      case EXCEPTION_INT_DIVIDE_BY_ZERO:
            S = "INT DIVIDE BY ZERO";
      break;
      //...
      }
      StatusBar1->Panels->Items[2]->Text =
            "Exception (" +
            (S.IsEmpty() ? (AnsiString)"" : "Code - " + S + ",") +
            "Address - "+
            IntToHex((int)Exception.ExceptionRecord.ExceptionAddress, 8)+")";
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerExitThreadDbgEvent(TObject *Sender,
      DWORD dwThreadId, EXIT_THREAD_DEBUG_INFO ExitThread, bool Pause)
{
      StatusBar1->Panels->Items[2]->Text = "Exit Thread";
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerInvalidBPCondition(TObject *Sender,
      AnsiString BPCondition)
{
      MessageDlg("Invalid Breakpoint Condition: " + BPCondition,
            mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerLoadDllDbgEvent(TObject *Sender,
      DWORD dwThreadId, LOAD_DLL_DEBUG_INFO LoadDll, bool Pause)
{
      void* Base = LoadDll.lpBaseOfDll;
      AnsiString Name;
      DWORD ImageSize;
      TDebugger::GetDllInfo(LoadDll.hFile, Name, ImageSize);
      char Path[MAX_PATH];

      Debugger->DbgProcess->Modules->EnumModules();
      TModule *Module = Debugger->DbgProcess->Modules->FindModule(Base);
      AnsiString DllPath = Module ? Module->Path : (AnsiString)Name;

      StatusBar1->Panels->Items[2]->Text =
            "Loading DLL: " + DllPath + " (Base Address - "+IntToHex((int)Base,8) + ")";

      TExports* Exports = ExportsList->AddExports(LoadDll.hFile);
      if(Exports)
            Exports->CorrectAddresses((DWORD)Base);
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerNeedStateShow(TObject *Sender,
      DWORD dwThreadId, bool OnlyContext)
{
      ShowCPUState(dwThreadId);
      if(OnlyContext) return;
      DisAsmView->ShowMemory( Debugger->GetEIP(dwThreadId) );
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerRipDbgEvent(TObject *Sender,
      DWORD dwThreadId, RIP_INFO RipInfo, AnsiString SystemMessage)
{
      StatusBar1->Panels->Items[2]->Text = "RIP Event (" + SystemMessage + ")";
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerThreadsListChange(TObject *Sender)
{
      // Update threads form
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerUnhandledException(TObject *Sender,
      EXCEPTION_RECORD ExceptionRecord, bool &bTerminateProcess)
{
      bTerminateProcess = MessageDlg(
            "Program generated Exception."
            "\n\nAt EIP = " + IntToHex((int)ExceptionRecord.ExceptionAddress,8)+
            (ExceptionRecord.ExceptionFlags ? "\n\nEXCEPTION NONCONTINUABLE" : "")+
            "\n\nThis Exception is NOT Being Handled"
            "\n\nTerminate Process?",
            mtError, mbOKCancel, 0) == mrOk;      
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerUnknownDebugEvent(TObject *Sender,
      DWORD dwThreadId)
{
      StatusBar1->Panels->Items[2]->Text = "Unknown debug event";
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerUnloadDllDbgEvent(TObject *Sender,
      DWORD dwThreadId, UNLOAD_DLL_DEBUG_INFO UnloadDll, bool Pause)
{
      StatusBar1->Panels->Items[2]->Text =
            "Unload DLL (Base Address - " +
            IntToHex((int)UnloadDll.lpBaseOfDll,8) + ")";

      ExportsList->Delete((DWORD)UnloadDll.lpBaseOfDll);
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DisAsmViewGutterClick(TObject *Sender, int X,
      int Y, int Line)
{
      if(DisAsmView->Buffer->ItemType[Line] == itAsmCode)
      {
            DWORD Address = DisAsmView->Buffer->ItemAddress[Line];
            if(Address)
            {
                  if(Debugger->CodeBP->Find(Address))
                  {
                        Debugger->CodeBP->Delete(Address);
                        return;
                  }
                  Debugger->CodeBP->Add(Address,CBP_NORMAL);
            }
      }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::CodeBPListChange(TObject *Sender,
      TExecBreakpointsListChangeType ChangeType,
      TExecBreakpointsListChangeInfo ChangeInfo)
{
      if(DisAsmView == NULL)
            return;
      switch(ChangeType)
      {
      case ctAdd:
      {
            int LineIndex = DisAsmView->GetLineByAddress(
                  ((TCodeBP*)ChangeInfo.AddedBP)->Address);
            DisAsmView->InvalidateGutterLine(LineIndex);
            DisAsmView->InvalidateLine(LineIndex);
      }
      break;
      case ctClear:
      case ctGroupedChange:
            DisAsmView->Invalidate();
      break;
      case ctDelete:
      {
            int LineIndex = DisAsmView->GetLineByAddress(ChangeInfo.DeletedBPAddress);
            DisAsmView->InvalidateGutterLine(LineIndex);
            DisAsmView->InvalidateLine(LineIndex);
      }
      break;
      case ctEdit:
      {
            int LineIndex = DisAsmView->GetLineByAddress(
                  ((TCodeBP*)ChangeInfo.EditedBP)->Address);
            if(ChangeInfo.Flags & EI_CHANGED_ENABLED)
            {
       	      DisAsmView->InvalidateGutterLine(LineIndex);
       	      DisAsmView->InvalidateLine(LineIndex);
            }
      }
      break;
      }
}
//---------------------------------------------------------------------------

void __fastcall TFMain::About1Click(TObject *Sender)
{
      TFAbout *FAbout = new TFAbout(Application);
      FAbout->ShowModal();
      delete FAbout;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerChangeCurrentInstr(TObject *Sender,
      TAsmCode AsmCode, DWORD EIP)
{
      TAsmOperandValue value;
      AnsiString S;
      if(Debugger->EvaluateOperand(AsmCode.Oper1, AsmCode.CodeLen, value))
            S = "Oper1=0x" + IntToHex((int)value.dwLowValue, 8);
      if(Debugger->EvaluateOperand(AsmCode.Oper2, AsmCode.CodeLen, value))
            S += "; Oper2=0x" + IntToHex((int)value.dwLowValue, 8);
      if(Debugger->EvaluateOperand(AsmCode.Oper3, AsmCode.CodeLen, value))
            S += "; Oper3=0x" + IntToHex((int)value.dwLowValue, 8);
      StatusBar1->Panels->Items[3]->Text = S;
}
//---------------------------------------------------------------------------

void __fastcall TFMain::DebuggerInternalError(TObject *Sender,
      AnsiString File, int Line, AnsiString Message)
{
      TFError *FError = new TFError(Application);
      FError->lblFileName->Caption = File.IsEmpty() ? (AnsiString)"<UNKNOWN>" : File;
      FError->lblLine->Caption = Line;
      FError->lblMessage->Caption = Message.IsEmpty() ? (AnsiString)"<UNKNOWN>" : Message;
      FError->ShowModal();
      delete FError;

      Debugger->CloseDebugger();
      ExportsList->Clear();
      DisAsmView->ProcessID = 0;
      DisAsmView->Buffer->Clear();
      DumpView->ProcessID = 0;
      DumpView->ShowMemory(0);
      StackView->ProcessID = 0;
      StackView->ShowMemory(0);

      edEAX->Text = "";
	edEBX->Text = "";
	edECX->Text = "";
	edEDX->Text = "";
	edESI->Text = "";
	edEDI->Text = "";
	edESP->Text = "";
	edEBP->Text = "";
	edEFLAGS->Text = "";
	edEIP->Text = "";

      StatusBar1->Panels->Items[0]->Text = "Stopped";
      StatusBar1->Panels->Items[1]->Text = "Module:";
      StatusBar1->Panels->Items[2]->Text = "";
      StatusBar1->Panels->Items[3]->Text = "";
}
//---------------------------------------------------------------------------

