/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*      Debugger Engine                                                       *
*      File: DataBPList.h                                                    *
*      Copyright (c) 2004-2005 Fysyuk Victor, <RockDbgSupport@ua.fm>         *
*      ALL RIGHTS RESERVED.                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#ifndef DataBPListH
#define DataBPListH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>
//---------------------------------------------------------------------------

enum TDebugRegister { DR0, DR1, DR2, DR3 };
enum TDataBPLen { bplBYTE, bplWORD, bplDWORD };

class PACKAGE TDataBP : public TPersistent
{
private:
      DWORD FAddress;
      DWORD FPassCount;
      bool FWriteOnly;
	bool FEnabled;
      AnsiString FGroup;
      AnsiString FLogMessage;
      AnsiString FEnableGroup;
      AnsiString FDisableGroup;
      DWORD FdwThreadID;
      TDataBPLen FBPLen;
      TNotifyEvent FOnChange;
      TDebugRegister FDebugRegister;
	AnsiString FCondition;
	AnsiString FEvalExpression;
      DWORD FCurrentPassCount;
      void __fastcall SetEnabled(bool value);
      void __fastcall SetPassCount(DWORD value);
      void __fastcall SetGroup(AnsiString value);
      void __fastcall SetLogMessage(AnsiString value);
      void __fastcall SetEnableGroup(AnsiString value);
      void __fastcall SetDisableGroup(AnsiString value);
	void __fastcall SetCondition(AnsiString value);
	void __fastcall SetEvalExpression(AnsiString value);
      void __fastcall SetCurrentPassCount(DWORD value);
public:
      TDataBP(DWORD ThreadID, DWORD dwAddress, TDataBPLen Len,TDebugRegister Reg,bool bWriteOnly);
      __fastcall ~TDataBP();
      __property DWORD Address  = { read=FAddress };
      __property bool WriteOnly  = { read=FWriteOnly };
      __property bool Enabled  = { read=FEnabled, write=SetEnabled };
      __property DWORD dwThreadID  = { read=FdwThreadID };
      __property TDebugRegister DebugRegister = { read=FDebugRegister };
      __property TDataBPLen BPLen = { read=FBPLen };
      __property DWORD PassCount  = { read=FPassCount, write=SetPassCount };
      __property DWORD CurrentPassCount  = { read=FCurrentPassCount, write=SetCurrentPassCount };
      __property AnsiString Group  = { read=FGroup, write=SetGroup };
      __property AnsiString LogMessage  = { read=FLogMessage, write=SetLogMessage };
      __property AnsiString EnableGroup  = { read=FEnableGroup, write=SetEnableGroup };
      __property AnsiString DisableGroup  = { read=FDisableGroup, write=SetDisableGroup };
      __property TNotifyEvent OnChange = { read=FOnChange, write=FOnChange };
	__property AnsiString Condition  = { read=FCondition, write=SetCondition };
	__property AnsiString EvalExpression  = { read=FEvalExpression, write=SetEvalExpression };
};

typedef void __fastcall (__closure *TDataBreakpointChangeEvent)(TObject* Sender);

class PACKAGE TDataBPList : public TComponent
{
private:
        TDataBP *FDataBP0;
        TDataBP *FDataBP1;
        TDataBP *FDataBP2;
        TDataBP *FDataBP3;
        BYTE FCount;
        TDataBreakpointChangeEvent FOnDataBreakpointChange;
        TDataBP* __fastcall GetItemByAddress(DWORD Address);
      void __fastcall BPChangeHandler(TObject* Sender);
protected:
public:
      __fastcall TDataBPList(TComponent* Owner);
      __fastcall ~TDataBPList();
      TDataBP* __fastcall Add(DWORD ThreadID, DWORD dwAddress, TDataBPLen Len, bool bWriteOnly);
      void __fastcall Delete(DWORD Address);
      void __fastcall Clear();
	TDataBP* __fastcall Find(DWORD Address) const;
      void EnableGroup(AnsiString GroupName);
      void DisableGroup(AnsiString GroupName);
      __property BYTE Count  = { read=FCount };
      __property TDataBP* DataBP0  = { read=FDataBP0 };
      __property TDataBP* DataBP1  = { read=FDataBP1 };
      __property TDataBP* DataBP2  = { read=FDataBP2 };
      __property TDataBP* DataBP3  = { read=FDataBP3 };
__published:
      __property TDataBreakpointChangeEvent OnDataBreakpointChange  = { read=FOnDataBreakpointChange, write=FOnDataBreakpointChange };
};
//---------------------------------------------------------------------------
#endif
 