/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*      Debugger Engine                                                       *
*      File: Debugger.h                                                      *
*      Copyright (c) 2004-2005 Fysyuk Victor, <RockDbgSupport@ua.fm>         *
*      ALL RIGHTS RESERVED.                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#ifndef DebuggerH
#define DebuggerH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>
#include <SyncObjs.hpp>
#include "Dialogs.hpp"
#include "CodeBPList.h"
#include "DataBPList.h"
#include "ExportsList.h"
#include "Modules.h"
#include "DisAsmEngine.h"
//---------------------------------------------------------------------------

struct TAsmOperandValue
{
        DWORD dwLowValue;
        DWORD dwHighValue;
};

struct TDbgThread
{
    HANDLE hThread;
    DWORD dwThreadID;
    LPTHREAD_START_ROUTINE lpStartAddress;
    int nPriority;
};

struct TThreadsListItem
{
      TDbgThread Thread;
      TThreadsListItem *Next;
};

class PACKAGE TThreadsList
{
private:
      TThreadsListItem *FItems;
      int FCount;
      TNotifyEvent FOnChange;
      TDbgThread* __fastcall GetItems(int Index);
public:
      TThreadsList();
      TDbgThread* __fastcall Add(HANDLE hThread, DWORD dwThreadID, LPTHREAD_START_ROUTINE lpStartAddress, int nPriority);
      void __fastcall Delete(DWORD dwThreadId);
      void __fastcall Clear();
      ~TThreadsList();
      __property int Count  = { read=FCount };
      __property TDbgThread* Items[int Index]  = { read=GetItems };
      __property TNotifyEvent OnChange  = { read=FOnChange, write=FOnChange };
};

class TDebugEventThread;
class TDebugger;

class TDebugEventThread : public TThread
{
friend TDebugger;
protected:
      void __fastcall Execute();
public:
      __fastcall TDebugEventThread(bool CreateSuspended);
      __fastcall ~TDebugEventThread();
      TDebugger *Debugger;
      TSimpleEvent *TerminateEvent;
};

class TDbgProcess
{
friend TDebugger;
friend TDebugEventThread;
private:
      DWORD	FdwProcessID;
      DWORD	FdwThreadID;
      HANDLE FhProcess;
      HANDLE FhFile;
      LPVOID FlpImage;
      DWORD	FdwDbgInfoOffset;
      DWORD	FnDbgInfoSize;
      TThreadsList FThreads;
      DWORD	 FdwPriority;
      AnsiString	 FModulePath;
      TModulesList* FModules;
      AnsiString FCommandLine;
public:
      TDbgProcess(AnsiString Path);
      __fastcall ~TDbgProcess();
      __property DWORD dwProcessID  = { read=FdwProcessID };
      __property DWORD dwThreadID  = { read=FdwThreadID };
      __property HANDLE hProcess  = { read=FhProcess };
      __property HANDLE hFile  = { read=FhFile };
      __property LPVOID lpImage  = { read=FlpImage };
      __property DWORD DbgInfoOffset  = { read=FdwDbgInfoOffset };
      __property DWORD DbgInfoSize  = { read=FnDbgInfoSize };
      __property TThreadsList Threads  = { read=FThreads };
      __property DWORD dwPriority  = { read=FdwPriority };
      __property AnsiString ModulePath  = { read=FModulePath };
      __property TModulesList* Modules  = { read=FModules };
      __property AnsiString CommandLine = { read=FCommandLine, write=FCommandLine };
};

enum TActionOnException {aoeStopAlways, aoeStopIfNotHandled};

enum TDebuggerState {dsStopped, dsRunning, dsPaused,
            dsStepOver, dsStepInto, dsRunUntilReturn};

typedef void __fastcall (__closure *TExceptionDbgEvent)(TObject *Sender, DWORD dwThreadId, EXCEPTION_DEBUG_INFO Exception);
typedef void __fastcall (__closure *TCreateThreadDbgEvent)(TObject *Sender, DWORD dwThreadId, CREATE_THREAD_DEBUG_INFO CreateThread, bool Pause);
typedef void __fastcall (__closure *TCreateProcessDbgEvent)(TObject *Sender, DWORD dwThreadId, CREATE_PROCESS_DEBUG_INFO CreateProcessInfo);
typedef void __fastcall (__closure *TExitThreadDbgEvent)(TObject *Sender, DWORD dwThreadId, EXIT_THREAD_DEBUG_INFO ExitThread, bool Pause);
typedef void __fastcall (__closure *TExitProcessDbgEvent)(TObject *Sender, DWORD dwThreadId, EXIT_PROCESS_DEBUG_INFO ExitProcess);
typedef void __fastcall (__closure *TLoadDllDbgEvent)(TObject *Sender, DWORD dwThreadId, LOAD_DLL_DEBUG_INFO LoadDll, bool Pause);
typedef void __fastcall (__closure *TUnloadDllDbgEvent)(TObject *Sender, DWORD dwThreadId, UNLOAD_DLL_DEBUG_INFO UnloadDll, bool Pause);
typedef void __fastcall (__closure *TDebugStringDbgEvent)(TObject *Sender, DWORD dwThreadId, AnsiString DebugString,OUTPUT_DEBUG_STRING_INFO DebugStringInfo, bool Pause);
typedef void __fastcall (__closure *TRipDbgEvent)(TObject *Sender, DWORD dwThreadId, RIP_INFO RipInfo, AnsiString SystemMessage);
typedef void __fastcall (__closure *TUnknownDebugEvent)(TObject *Sender, DWORD dwThreadId);
typedef void __fastcall (__closure *TUnhandledExceptionEvent)(TObject *Sender, EXCEPTION_RECORD ExceptionRecord, bool &bTerminateProcess);

typedef void __fastcall (__closure *TCodeBPDetectedDbgEvent)(TObject *Sender, DWORD dwThreadId, TCodeBP *CodeBP);
typedef void __fastcall (__closure *TDataBPDetectedDbgEvent)(TObject *Sender, DWORD dwThreadId, TDataBP *DataBP);
typedef void __fastcall (__closure *TInvalidBPConditionEvent)(TObject *Sender, AnsiString BPCondition);

typedef void __fastcall (__closure *TChangeDebuggerState)(TObject *Sender, DWORD dwThreadId, TDebuggerState NewState);
typedef void __fastcall (__closure *TNeedStateShow)(TObject *Sender, DWORD dwThreadId, bool OnlyContext);
typedef void __fastcall (__closure *TChangeCurrentInstrEvent)(TObject *Sender, TAsmCode AsmCode, DWORD EIP);

typedef void __fastcall (__closure *TNamedCALLDbgEvent)(TObject *Sender, AnsiString FuncName, DWORD StackPtr);

typedef void __fastcall (__closure *TInternalErrorEvent)(TObject *Sender, AnsiString File, int Line, AnsiString Message);

typedef bool (*TEnumSEHchainFunc)(DWORD dwThreadId, DWORD Address, DWORD SEHandler,LPVOID Data);

class PACKAGE TDebugger : public TComponent
{
friend TDebugEventThread;
private:
      void __fastcall ThreadsListChangeHandler(TObject * Sender);
      void __fastcall StepOnCALL();
      void __fastcall DoThreadsListChange();
      void __fastcall DoExceptionDbgEvent();
      void __fastcall DoCreateThreadDbgEvent();
      void __fastcall DoCreateProcessDbgEvent();
      void __fastcall DoExitThreadDbgEvent();
      void __fastcall DoExitProcessDbgEvent();
      void __fastcall DoLoadDllDbgEvent();
      void __fastcall DoUnloadDllDbgEvent();
      void __fastcall DoDebugStringDbgEvent();
      void __fastcall DoRipDbgEvent();
      void __fastcall DoUnknownDebugEvent();
      void __fastcall DoNeedStateShow();
      void __fastcall DoChangeCurrentInstr();
	void __fastcall DoCodeBPDetectedDbgEvent();
      void __fastcall DoDataBPDetectedDbgEvent();
      void __fastcall DoUnhandledExceptionEvent();
      void __fastcall DoInvalidBPConditionEvent();
      void __fastcall DoInternalError();
      void __fastcall CodeBPHandler(DEBUG_EVENT& de, TCodeBP* BP, bool& StopOnBP);
      void __fastcall DataBPHandler(DWORD dwThreadId, TDataBP* BP, bool &StopOnBP);

      void __fastcall RestoreBreakpoint();
      bool __fastcall AutoStepping(DEBUG_EVENT& de);
      bool __fastcall CheckRunUntilRetEnd(DEBUG_EVENT& de, TContext *Context, int Parameter);
      void __fastcall DemoMessage();

      AnsiString FErrorFile;
      int FErrorLine;
      AnsiString FErrorMessage;
      AnsiString FInvalidBPCondition;
      bool FPrevReportedException;
      bool FTerminateProcess;
      DWORD FEntryPoint;
	TCodeBP *FDetectedCodeBP;
      TDataBP *FDetectedDataBP;
      bool FAlreadyStoppedOnEntryPoint;
      TAsmCode FCurrentInstr;
      DWORD FCurrentEIP;
      DWORD FRestoreBP;
      AnsiString FSystemMessage;
      AnsiString FDebugString;
      AnsiString FCommandLine;
      BYTE FCALLInstrLen;
      TAsmOperand FCALLOperand;
      bool FPause;
      TEvent *FAutoResetEvent;
      TEvent *FManualResetEvent;
      TEvent *FStoppedLockEvent;

      bool FWasPaused;
      int FSuspendCount;
      bool FOnlyContext;
      TDebuggerState FDebuggerState;
      TDebuggerState FWantedState;
      TDbgProcess* FDbgProcess;
      DWORD FCurrentThreadId;
      bool FDebugActiveProcess;
      TExceptionDbgEvent FOnExceptionDbgEvent;
      TCreateThreadDbgEvent FOnCreateThreadDbgEvent;
      TCreateProcessDbgEvent FOnCreateProcessDbgEvent;
      TExitThreadDbgEvent FOnExitThreadDbgEvent;
      TExitProcessDbgEvent FOnExitProcessDbgEvent;
      TLoadDllDbgEvent FOnLoadDllDbgEvent;
      TUnloadDllDbgEvent FOnUnloadDllDbgEvent;
      TDebugStringDbgEvent FOnDebugStringDbgEvent;
      TRipDbgEvent FOnRipDbgEvent;
      TUnknownDebugEvent FOnUnknownDebugEvent;
      TChangeDebuggerState FOnChangeDebuggerState;
      TNeedStateShow FOnNeedStateShow;
      TCodeBPList* FCodeBP;
      TDataBPList* FDataBP;
      TExportsList* FExportsList;
      void SuspendDebuggeeProcess();
      void ResumeDebuggeeProcess();
      void __fastcall SetDebuggerState(TDebuggerState value);
      void __fastcall SetCodeBP(TCodeBPList* value);
      void __fastcall SetDataBP(TDataBPList* value);
      void __fastcall SetExportsList(TExportsList* value);
      TDebugEventThread *FDebugEventThread;
      DEBUG_EVENT FDebugEvent;
      TNotifyEvent FOnThreadsListChange;
      TChangeCurrentInstrEvent FOnChangeCurrentInstr;
	TCodeBPDetectedDbgEvent FOnCodeBPDetected;
	TDataBPDetectedDbgEvent FOnDataBPDetected;
      TNamedCALLDbgEvent FOnNamedCALL;
      TUnhandledExceptionEvent FOnUnhandledException;
      TInvalidBPConditionEvent FOnInvalidBPCondition;
      TInternalErrorEvent FOnInternalError;
      bool FBreakOnCreateThread;
      bool FBreakOnExitThread;
      bool FBreakOnLoadDLL;
      bool FBreakOnUnloadDLL;
      bool FBreakOnDebugString;
      bool FTerminate;
      TActionOnException FExcptAccessViolation;
      TActionOnException FExcptArrayBoundsExceeded;
      TActionOnException FExcptDatatypeMisalignment;
      TActionOnException FExcptFloatDenormalOperand;
      TActionOnException FExcptFloatDivideByZero;
      TActionOnException FExcptFloatInexactResult;
      TActionOnException FExcptFloatInvalidOperation;
      TActionOnException FExcptFloatOverflow;
      TActionOnException FExcptFloatStackCheck;
      TActionOnException FExcptFloatUnderflow;
      TActionOnException FExcptIllegalInstruction;
      TActionOnException FExcptInPageError;
      TActionOnException FExcptIntDivideByZero;
      TActionOnException FExcptIntOverflow;
      TActionOnException FExcptInvalidDisposition;
      TActionOnException FExcptNoncontinuableException;
      TActionOnException FExcptPrivInstruction;
      TActionOnException FExcptStackOverflow;
      TActionOnException FExcptControlC;
      bool FDebugChildProcesses;
      void __fastcall ChangeDebuggerState();
      void __fastcall SetCurrentThreadId(DWORD value);
      bool __fastcall DebugEventLoop();
      void __fastcall DebugEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      BOOL __fastcall WaitForDbgEvent(LPDEBUG_EVENT lpDebugEvent, DWORD dwMilliseconds);
      BOOL __fastcall ContinueDbgEvent(DWORD dwThreadId, DWORD dwContinueStatus);

      void __fastcall ExceptionEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      void __fastcall CreateProcessEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      void __fastcall CreateThreadEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      void __fastcall ExitProcessEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      void __fastcall ExitThreadEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      void __fastcall RipEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      void __fastcall LoadDLLEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      void __fastcall UnloadDLLEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
      void __fastcall OutputDbgStringEventHandler(DEBUG_EVENT& de, DWORD& dwContinueStatus, bool& NeedPause);
protected:
      virtual void __fastcall Notification(TComponent* AComponent, TOperation Operation);
public:
      __fastcall TDebugger(TComponent* Owner);
      void __fastcall StartDebugger(AnsiString ModulePath, AnsiString CommandLine);
      void __fastcall StartDebugger(DWORD dwProcessId, AnsiString CommandLine);//For DebugActiveProcess
      void __fastcall CloseDebugger();
      void __fastcall Pause();
      void __fastcall Run();
      void __fastcall RunUntilReturn();
      void __fastcall StepOver();
      void __fastcall StepInto();
      bool __fastcall CanPause();
      bool __fastcall CanRun();
      bool __fastcall CanStep();
      void __fastcall Terminate();
      DWORD __fastcall Find(DWORD Address, DWORD Size, BYTE *Pattern, DWORD PatternSize);
      static bool GetDllInfo(HANDLE hDllfile, AnsiString &DllName, DWORD &ImageSize);
      bool __fastcall ReadMemory(DWORD dwProcessId, LPCVOID lpBaseAddress, LPVOID lpBuffer, DWORD nSize);
      bool __fastcall ReadMemory(LPCVOID lpBaseAddress, LPVOID lpBuffer, DWORD nSize)
      {
            if(DbgProcess == NULL) return false;
            return ReadMemory(DbgProcess->dwProcessID, lpBaseAddress, lpBuffer, nSize);
      }
      bool __fastcall WriteMemory(DWORD dwProcessId, LPVOID lpBaseAddress, LPVOID lpBuffer, DWORD nSize);
      bool __fastcall WriteMemory(LPVOID lpBaseAddress, LPVOID lpBuffer, DWORD nSize)
      {
            if(DbgProcess == NULL) return false;
            return WriteMemory(DbgProcess->dwProcessID, lpBaseAddress, lpBuffer, nSize);
      }
      bool __fastcall EvaluateGeneralRegister(DWORD dwThreadId,TRegister Register,DWORD &Value);
      bool __fastcall EvaluateGeneralRegister(TRegister Register,DWORD &Value)
      {
            return EvaluateGeneralRegister(CurrentThreadId, Register, Value);
      }
      bool __fastcall EvaluateSegmentRegister(DWORD dwThreadId,TRegister Register,DWORD &Value);
      bool __fastcall EvaluateSegmentRegister(TRegister Register,DWORD &Value)
      {
            return EvaluateSegmentRegister(CurrentThreadId, Register, Value);
      }
      bool __fastcall EvaluateDebugRegister(DWORD dwThreadId,TRegister Register,DWORD &Value);
      bool __fastcall EvaluateDebugRegister(TRegister Register,DWORD &Value)
      {
            return EvaluateDebugRegister(CurrentThreadId, Register, Value);
      }
      bool __fastcall EvaluateRegister(DWORD dwThreadId,TRegister Register,DWORD &Value);
      bool __fastcall EvaluateRegister(TRegister Register,DWORD &Value)
      {
            return EvaluateRegister(CurrentThreadId, Register, Value);
      }
      bool __fastcall EvaluateOperand(DWORD dwThreadId,TAsmOperand Operand,
            BYTE InstrLen,TAsmOperandValue &Value);
      bool __fastcall EvaluateOperand(TAsmOperand Operand,
            BYTE InstrLen,TAsmOperandValue &Value)
      {
            return EvaluateOperand(CurrentThreadId, Operand, InstrLen, Value);
      }
      bool __fastcall GetContext(DWORD dwThreadId, DWORD ContextFlags, CONTEXT &Context);
      bool __fastcall GetContext(DWORD ContextFlags, CONTEXT &Context)
      {
            return GetContext(CurrentThreadId, ContextFlags, Context);
      }
      bool __fastcall SetContext(DWORD dwThreadId, DWORD ContextFlags, CONTEXT Context, bool NeedStateShow);
      bool __fastcall SetContext(DWORD ContextFlags, CONTEXT Context, bool NeedStateShow)
      {
            return SetContext(CurrentThreadId, ContextFlags, Context, NeedStateShow);
      }
      DWORD __fastcall GetEIP(DWORD dwThreadId);
      DWORD __fastcall GetEIP()
      {
            return GetEIP(CurrentThreadId);
      }
	bool __fastcall GetThreadSelEntry(DWORD dwThreadId, DWORD dwSelector, LDT_ENTRY &SelectorEntry) const;
      bool __fastcall GetThreadSelEntry(DWORD dwSelector, LDT_ENTRY &SelectorEntry) const
      {
            return GetThreadSelEntry(CurrentThreadId, dwSelector, SelectorEntry);
      }
      void __fastcall EnumSEHchain(DWORD dwThreadId, TEnumSEHchainFunc Func, LPVOID Data);
      void __fastcall EnumSEHchain(TEnumSEHchainFunc Func, LPVOID Data)
      {
            return EnumSEHchain(CurrentThreadId, Func, Data);
      }
      bool __fastcall GetInstructionByAddress(DWORD Address, TAsmCode& res);
      __property TDebuggerState DebuggerState  = { read=FDebuggerState, write=SetDebuggerState };
      __property TDbgProcess* DbgProcess  = { read=FDbgProcess };
      __property DWORD CurrentThreadId  = { read=FCurrentThreadId, write=SetCurrentThreadId };
__published:
      __property TCodeBPList* CodeBP  = { read=FCodeBP, write=SetCodeBP };
      __property TDataBPList* DataBP  = { read=FDataBP, write=SetDataBP };
      __property TExportsList* ExportsList  = { read=FExportsList, write=SetExportsList };
      __property TExceptionDbgEvent OnExceptionDbgEvent  = { read=FOnExceptionDbgEvent, write=FOnExceptionDbgEvent };
      __property TCreateThreadDbgEvent OnCreateThreadDbgEvent  = { read=FOnCreateThreadDbgEvent, write=FOnCreateThreadDbgEvent };
      __property TCreateProcessDbgEvent OnCreateProcessDbgEvent  = { read=FOnCreateProcessDbgEvent, write=FOnCreateProcessDbgEvent };
      __property TExitThreadDbgEvent OnExitThreadDbgEvent  = { read=FOnExitThreadDbgEvent, write=FOnExitThreadDbgEvent };
      __property TExitProcessDbgEvent OnExitProcessDbgEvent  = { read=FOnExitProcessDbgEvent, write=FOnExitProcessDbgEvent };
      __property TLoadDllDbgEvent OnLoadDllDbgEvent  = { read=FOnLoadDllDbgEvent, write=FOnLoadDllDbgEvent };
      __property TUnloadDllDbgEvent OnUnloadDllDbgEvent  = { read=FOnUnloadDllDbgEvent, write=FOnUnloadDllDbgEvent };
      __property TDebugStringDbgEvent OnDebugStringDbgEvent  = { read=FOnDebugStringDbgEvent, write=FOnDebugStringDbgEvent };
      __property TRipDbgEvent OnRipDbgEvent  = { read=FOnRipDbgEvent, write=FOnRipDbgEvent };
      __property TUnknownDebugEvent OnUnknownDebugEvent  = { read=FOnUnknownDebugEvent, write=FOnUnknownDebugEvent };
      __property TChangeDebuggerState OnChangeDebuggerState  = { read=FOnChangeDebuggerState, write=FOnChangeDebuggerState };
      __property TNotifyEvent OnThreadsListChange  = { read=FOnThreadsListChange, write=FOnThreadsListChange };
      __property TNeedStateShow OnNeedStateShow  = { read=FOnNeedStateShow, write=FOnNeedStateShow };
      __property TChangeCurrentInstrEvent OnChangeCurrentInstr  = { read=FOnChangeCurrentInstr, write=FOnChangeCurrentInstr };
	__property TCodeBPDetectedDbgEvent OnCodeBPDetected  = { read=FOnCodeBPDetected, write=FOnCodeBPDetected };
      __property TDataBPDetectedDbgEvent OnDataBPDetected  = { read=FOnDataBPDetected, write=FOnDataBPDetected };
      __property TNamedCALLDbgEvent OnNamedCALL  = { read=FOnNamedCALL, write=FOnNamedCALL };
      __property TUnhandledExceptionEvent OnUnhandledException  = { read=FOnUnhandledException, write=FOnUnhandledException };
      __property TInvalidBPConditionEvent OnInvalidBPCondition  = { read=FOnInvalidBPCondition, write=FOnInvalidBPCondition };
      __property TInternalErrorEvent OnInternalError  = { read=FOnInternalError, write=FOnInternalError };
      __property bool BreakOnCreateThread  = { read=FBreakOnCreateThread, write=FBreakOnCreateThread };
      __property bool BreakOnExitThread  = { read=FBreakOnExitThread, write=FBreakOnExitThread };
      __property bool BreakOnLoadDLL  = { read=FBreakOnLoadDLL, write=FBreakOnLoadDLL };
      __property bool BreakOnUnloadDLL  = { read=FBreakOnUnloadDLL, write=FBreakOnUnloadDLL };
      __property bool BreakOnDebugString  = { read=FBreakOnDebugString, write=FBreakOnDebugString };
      __property bool DebugChildProcesses  = { read=FDebugChildProcesses, write=FDebugChildProcesses };
      __property TActionOnException ExcptAccessViolation  = { read=FExcptAccessViolation, write=FExcptAccessViolation, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptArrayBoundsExceeded  = { read=FExcptArrayBoundsExceeded, write=FExcptArrayBoundsExceeded, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptDatatypeMisalignment  = { read=FExcptDatatypeMisalignment, write=FExcptDatatypeMisalignment, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptFloatDenormalOperand  = { read=FExcptFloatDenormalOperand, write=FExcptFloatDenormalOperand, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptFloatDivideByZero  = { read=FExcptFloatDivideByZero, write=FExcptFloatDivideByZero, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptFloatInexactResult  = { read=FExcptFloatInexactResult, write=FExcptFloatInexactResult, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptFloatInvalidOperation  = { read=FExcptFloatInvalidOperation, write=FExcptFloatInvalidOperation, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptFloatOverflow  = { read=FExcptFloatOverflow, write=FExcptFloatOverflow, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptFloatStackCheck  = { read=FExcptFloatStackCheck, write=FExcptFloatStackCheck, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptFloatUnderflow  = { read=FExcptFloatUnderflow, write=FExcptFloatUnderflow, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptIllegalInstruction  = { read=FExcptIllegalInstruction, write=FExcptIllegalInstruction, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptInPageError  = { read=FExcptInPageError, write=FExcptInPageError, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptIntDivideByZero  = { read=FExcptIntDivideByZero, write=FExcptIntDivideByZero, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptIntOverflow  = { read=FExcptIntOverflow, write=FExcptIntOverflow, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptInvalidDisposition  = { read=FExcptInvalidDisposition, write=FExcptInvalidDisposition, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptNoncontinuableException  = { read=FExcptNoncontinuableException, write=FExcptNoncontinuableException, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptPrivInstruction  = { read=FExcptPrivInstruction, write=FExcptPrivInstruction, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptStackOverflow  = { read=FExcptStackOverflow, write=FExcptStackOverflow, default=aoeStopIfNotHandled };
      __property TActionOnException ExcptControlC  = { read=FExcptControlC, write=FExcptControlC, default=aoeStopIfNotHandled };
};
//---------------------------------------------------------------------------

#endif
