/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*      Disassembler Engine.                                                  *
*      Copyright (c) 2004-2005 Fysyuk Victor, <RockDbgSupport@ua.fm>         *
*      ALL RIGHTS RESERVED.                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
#ifndef DisAsmEngineH
#define DisAsmEngineH
#include <windows.h>
//---------------------------------------------------------------------------
enum TOperand
{
        opeAX,opeCX,opeDX,opeBX,opeSP,opeBP,opeSI,opeDI,
        opEAX,opECX,opEDX,opEBX,opESP,opEBP,opESI,opEDI,
        opAL,opAH,opAX,opCL,opCH,opBL,opBH,opDL,opDH,opDX,
        opCS,opDS,opSS,opES,opFS,opGS,
        opAp,opCd,opDd,opEb,opEd,opEp,opEv,opEw,opGb,opGd,opGv,opGw,
        opIb,opIv,opIw,opJb,opJv,opM,opMa,opMb,opMp,opMq,opMs,opOb,opOv,
        opPd,opPdq,opPpi,opPq,opQd,opQdq,opQpi,opQq,opRd,opSw,opTd,
        opVdq,opVpd,opVps,opVq,opVss,
        opWdq,opWpd,opWps,opWq,opWs,opWss,opWsd,
        opXb,opXv,opYb,opYv,opVsd,
        op_3,op_1,
        opNone,opFv,
        //For FPU instruction
        opSingleReal,opDoubleReal,opExtendedReal,
        opWordInteger,opDwordInteger,opQwordInteger,opPackedBCD,
        opFPU98_108,opFPU14_28,opFPU_2,
        opST0,opST1,opST2,opST3,opST4,opST5,opST6,opST7,
        //For Table_OpCodeExt
        opIb_Iv,opAL_eAX
};

enum TRegister
{
      rNONE, // must be first!
      rEAX,rECX,rEBX,rEDX,rESI,rEDI,rESP,rEBP,
      rAX,rCX,rBX,rDX,rSI,rDI,rSP,rBP,
      rAH,rCH,rBH,rDH,rAL,rCL,rBL,rDL,
      rCS,rDS,rES,rFS,rGS,rSS,
      rCR0,rCR2,rCR3,rCR4,
      rDR0,rDR1,rDR2,rDR3,rDR6,rDR7,
      rST0,rST1,rST2,rST3,rST4,rST5,rST6,rST7,
      rMM0,rMM1,rMM2,rMM3,rMM4,rMM5,rMM6,rMM7,
      rXMM0,rXMM1,rXMM2,rXMM3,rXMM4,rXMM5,rXMM6,rXMM7,
      rERROR,rEIP
};

//aopRelativeOffset -  ,      IP/EIP
enum TAsmOperType { aopReg, aopMemory, aopRelativeOffset,aopFarPtr, aopImm, aopError, aopNone };
enum TAddrMode { am16bit, am32bit };
enum TOperSize { os16bit, os32bit };
enum TRegType { rtGeneral, rtMMX, rtXMM };

struct TAsmOperand  //Example: byte ptr cs:[ecx+4*dx+12345]
{
        TAsmOperType OperType;
        unsigned char Length;  //Length=8 => byte ptr
        TRegister OverrSegm;
        TRegister Base_or_Reg;  //Base register or Register
        TRegister Index;
        unsigned char Scale;
        long Displ_or_Imm;      //Disp. or Immed. value
        BYTE DisplSize; //Disp. Size - 8, 16 or 32 bits
        int Segment; //Example: 9A475600002001 - call 00120:000005647
};

struct TOpCode
{
        unsigned int MnmIndex;  //index in Mnemonics array
        TOperand Operand1,Operand2,Operand3;
};

struct TInstr
{
        BYTE Code[15];  //Code[0] - length of command
        BYTE COPLen,ModRMLen,SIBLen,OffsetLen,ImmLen;
        BYTE Offset; //Instruction offset in Code[]
};

#define PREFIX_SEGMENT_OVERRIDE		0x01
#define PREFIX_LOCK                       0x02
#define PREFIX_REPNE                      0x04
#define PREFIX_REP 				0x08	//REP or REPE
#define PREFIX_BRANCH_HINTS               0x10
#define PREFIX_OPERANDSIZE_OVERRIDE       0x20
#define PREFIX_ADDRESSSIZE_OVERRIDE       0x40

struct TAsmCode
{
        unsigned int MnmIndex; //index in Mnemonics array
        TAsmOperand Oper1,Oper2,Oper3;
        BYTE CodeLen;
        BYTE Prefixes; //flags PREFIX_xxx
};

enum
{
      MNEM_2_BYTE_ESCAPE,
	MNEM_2_BYTE_ILLEGAL_OPCODES_UD2_1B,
	MNEM_AAA,
	MNEM_AAD,
	MNEM_AAM,
	MNEM_AAS,
	MNEM_ADC,
	MNEM_ADD,
	MNEM_ADDPD,
	MNEM_ADDPS,
	MNEM_ADDRSIZEPREFIX,
	MNEM_ADDSD,
	MNEM_ADDSS,
	MNEM_AND,
	MNEM_ANDNPD,
	MNEM_ANDNPS,
	MNEM_ANDPD,
	MNEM_ANDPS,
	MNEM_ARPL,
	MNEM_BOUND,
	MNEM_BSF,
	MNEM_BSR,
	MNEM_BSWAP,
	MNEM_BT,
	MNEM_BTC,
	MNEM_BTR,
	MNEM_BTS,
	MNEM_CALL,
	MNEM_CALLF,
	MNEM_CALLN,

	MNEM_CBW_CWDE,
        MNEM_CBW,
        MNEM_CWDE,

	MNEM_CLC,
	MNEM_CLD,
	MNEM_CLFLUSH,
	MNEM_CLI,
	MNEM_CLTS,
	MNEM_CMC,
	MNEM_CMOV_A_NBE,
	MNEM_CMOV_AE_NB_NC,
	MNEM_CMOV_B_C_NAE,
	MNEM_CMOV_BE_NA,
	MNEM_CMOV_E_Z,
	MNEM_CMOV_L_NGE,
	MNEM_CMOV_LE_NG,
	MNEM_CMOV_NE_NZ,
	MNEM_CMOV_NL_GE,
	MNEM_CMOV_NLE_G,
	MNEM_CMOV_NP_PO,
	MNEM_CMOV_P_PE,
	MNEM_CMOVNO,
	MNEM_CMOVNS,
	MNEM_CMOVO,
	MNEM_CMOVS,
	MNEM_CMP,
	MNEM_CMPPD,
	MNEM_CMPPS,
	MNEM_CMPSB,
	MNEM_CMPSS,

	MNEM_CMPSW_CMPSD,
        MNEM_CMPSW,
        MNEM_CMPSD,

	MNEM_CMPXCHG,
	MNEM_CMPXCHG8B,
	MNEM_COMISD,
	MNEM_COMISS,
	MNEM_CPUID,
	MNEM_CS_,
	MNEM_CVTDQ2PD,
	MNEM_CVTDQ2PS,
	MNEM_CVTPD2DQ,
	MNEM_CVTPD2PI,
	MNEM_CVTPD2PS,
	MNEM_CVTPI2PD,
	MNEM_CVTPI2PS,
	MNEM_CVTPS2DQ,
	MNEM_CVTPS2PD,
	MNEM_CVTPS2PI,
	MNEM_CVTSD2SI,
	MNEM_CVTSD2SS,
	MNEM_CVTSI2SD,
	MNEM_CVTSI2SS,
	MNEM_CVTSS2SD,
	MNEM_CVTSS2SI,
	MNEM_CVTTPD2DQ,
	MNEM_CVTTPD2PI,
	MNEM_CVTTPS2DQ,
	MNEM_CVTTPS2PI,
	MNEM_CVTTSD2SI,
	MNEM_CVTTSS2SI,

	MNEM_CWD_CDQ,
        MNEM_CWD,
        MNEM_CDQ,

	MNEM_DAA,
	MNEM_DAS,
	MNEM_DEC,
	MNEM_DIV,
	MNEM_DIVPD,
	MNEM_DIVPS,
	MNEM_DIVSD,
	MNEM_DIVSS,
	MNEM_DS_,
	MNEM_EMMS,
	MNEM_ENTER,
	MNEM_ERROR,
	MNEM_ES_,
	MNEM_ESC,
	MNEM_F2XM1,
	MNEM_FABS,
	MNEM_FADD,
	MNEM_FADDP,
	MNEM_FBLD,
	MNEM_FBSTP,
	MNEM_FCHS,
	MNEM_FCLEX,
	MNEM_FCMOVB,
	MNEM_FCMOVBE,
	MNEM_FCMOVE,
	MNEM_FCMOVNB,
	MNEM_FCMOVNBE,
	MNEM_FCMOVNE,
	MNEM_FCMOVNU,
	MNEM_FCMOVU,
	MNEM_FCOM,
	MNEM_FCOMI,
	MNEM_FCOMIP,
	MNEM_FCOMP,
	MNEM_FCOMPP,
	MNEM_FCOS,
	MNEM_FDECSTP,
	MNEM_FDIV,
	MNEM_FDIVP,
	MNEM_FDIVR,
	MNEM_FDIVRP,
	MNEM_FFREE,
	MNEM_FIADD,
	MNEM_FICOM,
	MNEM_FICOMP,
	MNEM_FIDIV,
	MNEM_FIDIVR,
	MNEM_FILD,
	MNEM_FIMUL,
	MNEM_FINCSTP,
	MNEM_FINIT,
	MNEM_FIST,
	MNEM_FISTP,
	MNEM_FISUB,
	MNEM_FISUBR,
	MNEM_FLD,
	MNEM_FLD1,
	MNEM_FLDCW,
	MNEM_FLDENV,
	MNEM_FLDL2E,
	MNEM_FLDL2T,
	MNEM_FLDLG2,
	MNEM_FLDLN2,
	MNEM_FLDPI,
	MNEM_FLDZ,
	MNEM_FMUL,
	MNEM_FMULP,
	MNEM_FNOP,
	MNEM_FPATAN,
	MNEM_FPREM,
	MNEM_FPREM1,
	MNEM_FPTAN,
	MNEM_FRNDINT,
	MNEM_FRSTOR,
	MNEM_FS_,
	MNEM_FSAVE,
	MNEM_FSCALE,
	MNEM_FSIN,
	MNEM_FSINCOS,
	MNEM_FSQRT,
	MNEM_FST,
	MNEM_FSTCW,
	MNEM_FSTENV,
	MNEM_FSTP,
	MNEM_FSTSW,
	MNEM_FSUB,
	MNEM_FSUBP,
	MNEM_FSUBR,
	MNEM_FSUBRP,
	MNEM_FTST,
	MNEM_FUCOM,
	MNEM_FUCOMI,
	MNEM_FUCOMIP,
	MNEM_FUCOMP,
	MNEM_FUCOMPP,
	MNEM_FXAM,
	MNEM_FXCH,
	MNEM_FXRSTOR,
	MNEM_FXSAVE,
	MNEM_FXTRACT,
	MNEM_FYL2X,
	MNEM_FYL2XP1,
	MNEM_GRP_10_1A_INVALID_OPCODE_1B,
	MNEM_GRP_11_1A___MOV,
	MNEM_GRP_12_1A,
	MNEM_GRP_13_1A,
	MNEM_GRP_14_1A,
	MNEM_GRP_15_1A_1C,
	MNEM_GRP_6_1A,
	MNEM_GRP_7_1A,
	MNEM_GRP_8_1A,
	MNEM_GRP_9_1A,
	MNEM_GS_,
	MNEM_HLT,
	MNEM_IDIV,
	MNEM_IMMEDIATE_GRP_1_1A,
	MNEM_IMUL,
	MNEM_IN,
	MNEM_INC,
	MNEM_INC_DEC_GRP_4_1A,
	MNEM_INC_DEC_GRP_5_1A,
	MNEM_INSB,

	MNEM_INSW_INSD,
        MNEM_INSW,
        MNEM_INSD,

	MNEM_INT,
	MNEM_INTO,
	MNEM_INVD,
	MNEM_INVLPG,
	MNEM_IRET,
	MNEM_J_A_NBE,
	MNEM_J_AE_NB_NC,
	MNEM_J_B_C_NAE,
	MNEM_J_BE_NA,
	MNEM_J_E_Z,
	MNEM_J_L_NGE,
	MNEM_J_LE_NG,
	MNEM_J_NE_NZ,
	MNEM_J_NL_GE,
	MNEM_J_NLE_G,
	MNEM_J_NP_PO,
	MNEM_J_P_PE,
	MNEM_JA,
	MNEM_JAE,
	MNEM_JB,
	MNEM_JBE,

	MNEM_JCXZ_JECXZ,
        MNEM_JCXZ,
        MNEM_JECXZ,

	MNEM_JE,
	MNEM_JG,
	MNEM_JGE,
	MNEM_JL,
	MNEM_JLE,
	MNEM_JMP_FAR,
	MNEM_JMP_NEAR,
	MNEM_JMP_SHORT,
	MNEM_JMPF,
	MNEM_JMPN,
	MNEM_JNE,
	MNEM_JNO,
	MNEM_JNP,
	MNEM_JNS,
	MNEM_JO,
	MNEM_JP,
	MNEM_JS,
	MNEM_LAHF,
	MNEM_LAR,
	MNEM_LDMXCSR,
	MNEM_LDS,
	MNEM_LEA,
	MNEM_LEAVE,
	MNEM_LES,
	MNEM_LFENCE,
	MNEM_LFS,
	MNEM_LGDT,
	MNEM_LGS,
	MNEM_LIDT,
	MNEM_LLDT,
	MNEM_LMSW,
	MNEM_LOCK,
	MNEM_LODSB,

	MNEM_LODSW_LODSD,
	MNEM_LODSW,
        MNEM_LODSD,

	MNEM_LOOP,
	MNEM_LOOPE,
	MNEM_LOOPNE,
	MNEM_LSL,
	MNEM_LSS,
	MNEM_LTR,
	MNEM_MASKMOVDQU,
	MNEM_MASKMOVQ,
	MNEM_MAXPD,
	MNEM_MAXPS,
	MNEM_MAXSD,
	MNEM_MAXSS,
	MNEM_MFENCE,
	MNEM_MINPD,
	MNEM_MINPS,
	MNEM_MINSD,
	MNEM_MINSS,
	MNEM_MMX_UD,
	MNEM_MOV,
	MNEM_MOVAPD,
	MNEM_MOVAPS,
	MNEM_MOVD,
	MNEM_MOVDQ2Q,
	MNEM_MOVDQA,
	MNEM_MOVDQU,
	MNEM_MOVHPD,
	MNEM_MOVHPS,
	MNEM_MOVLPD,
	MNEM_MOVLPS,
	MNEM_MOVMSKPD,
	MNEM_MOVMSKPS,
	MNEM_MOVNTDQ,
	MNEM_MOVNTI,
	MNEM_MOVNTPD,
	MNEM_MOVNTPS,
	MNEM_MOVNTQ,
	MNEM_MOVQ,
	MNEM_MOVQ2DQ,
	MNEM_MOVSB,
	MNEM_MOVSS,

	MNEM_MOVSW_MOVSD,
        MNEM_MOVSW,
        MNEM_MOVSD,

	MNEM_MOVSX,
	MNEM_MOVUPD,
	MNEM_MOVUPS,
	MNEM_MOVZX,
	MNEM_MUL,
	MNEM_MULPD,
	MNEM_MULPS,
	MNEM_MULSD,
	MNEM_MULSS,
	MNEM_NEG,
	MNEM_NOP,
	MNEM_NOT,
	MNEM_OPDSIZEPREFIX,
	MNEM_OR,
	MNEM_ORPD,
	MNEM_ORPS,
	MNEM_OUT,
	MNEM_OUTSB,

	MNEM_OUTSW_OUTSD,
        MNEM_OUTSW,
        MNEM_OUTSD,

	MNEM_PACKSSDW,
	MNEM_PACKSSWB,
	MNEM_PACKUSWB,
	MNEM_PADDB,
	MNEM_PADDD,
	MNEM_PADDQ,
	MNEM_PADDSB,
	MNEM_PADDSW,
	MNEM_PADDUSB,
	MNEM_PADDUSW,
	MNEM_PADDW,
	MNEM_PAND,
	MNEM_PANDN,
	MNEM_PAVGB,
	MNEM_PAVGW,
	MNEM_PCMPEQB,
	MNEM_PCMPEQD,
	MNEM_PCMPEQW,
	MNEM_PCMPGTB,
	MNEM_PCMPGTD,
	MNEM_PCMPGTW,
	MNEM_PCMPQTD,
	MNEM_PEXTRW,
	MNEM_PINSRW,
	MNEM_PMADDWD,
	MNEM_PMAXSW,
	MNEM_PMAXUB,
	MNEM_PMINSW,
	MNEM_PMINUB,
	MNEM_PMOVMSKB,
	MNEM_PMULHUW,
	MNEM_PMULHW,
	MNEM_PMULLW,
	MNEM_PMULUDQ,
	MNEM_POP,

	MNEM_POPA_POPAD,
        MNEM_POPA,
        MNEM_POPAD,

	MNEM_POPF_POPFD,
        MNEM_POPF,
        MNEM_POPFD,

	MNEM_POR,
	MNEM_PREFETCH_1C__GRP_16_1A__,
	MNEM_PREFETCHNTA,
	MNEM_PREFETCHT0,
	MNEM_PREFETCHT1,
	MNEM_PREFETCHT2,
	MNEM_PSADBW,
	MNEM_PSHUFD,
	MNEM_PSHUFHW,
	MNEM_PSHUFLW,
	MNEM_PSHUFW,
	MNEM_PSLLD,
	MNEM_PSLLDQ,
	MNEM_PSLLQ,
	MNEM_PSLLW,
	MNEM_PSRAD,
	MNEM_PSRAW,
	MNEM_PSRLD,
	MNEM_PSRLDQ,
	MNEM_PSRLQ,
	MNEM_PSRLW,
	MNEM_PSUBB,
	MNEM_PSUBD,
	MNEM_PSUBQ,
	MNEM_PSUBSB,
	MNEM_PSUBSW,
	MNEM_PSUBUSB,
	MNEM_PSUBUSW,
	MNEM_PSUBW,
	MNEM_PUNPCKHBW,
	MNEM_PUNPCKHDQ,
	MNEM_PUNPCKHQDQ,
	MNEM_PUNPCKHWD,
	MNEM_PUNPCKLBW,
	MNEM_PUNPCKLDQ,
	MNEM_PUNPCKLQDQ,
	MNEM_PUNPCKLWD,
	MNEM_PUSH,

	MNEM_PUSHA_PUSHAD,
        MNEM_PUSHA,
        MNEM_PUSHAD,

	MNEM_PUSHF_PUSHFD,
        MNEM_PUSHF,
        MNEM_PUSHFD,

	MNEM_PXOR,
	MNEM_RCL,
	MNEM_RCPPS,
	MNEM_RCPSS,
	MNEM_RCR,
	MNEM_RDMSR,
	MNEM_RDPMC,
	MNEM_RDTSC,
	MNEM_REP_REPE,
	MNEM_REPNE,
	MNEM_RET,
	MNEM_RETF,
	MNEM_ROL,
	MNEM_ROR,
	MNEM_RSM,
	MNEM_RSQRTPS,
	MNEM_RSQRTSS,
	MNEM_SAHF,
	MNEM_SAR,
	MNEM_SBB,
	MNEM_SCASB,

	MNEM_SCASW_SCASD,
        MNEM_SCASW,
        MNEM_SCASD,

	MNEM_SET_A_NBE,
	MNEM_SET_AE_NB_NC,
	MNEM_SET_B_C_NAE,
	MNEM_SET_BE_NA,
	MNEM_SET_E_Z,
	MNEM_SET_L_NGE,
	MNEM_SET_LE_NG,
	MNEM_SET_NE_NZ,
	MNEM_SET_NL_GE,
	MNEM_SET_NLE_G,
	MNEM_SET_NP_PO,
	MNEM_SET_P_PE,
	MNEM_SETNO,
	MNEM_SETNS,
	MNEM_SETO,
	MNEM_SETS,
	MNEM_SGDT,
	MNEM_SHIFT_GRP_2_1A,
	MNEM_SHL,
	MNEM_SHLD,
	MNEM_SHR,
	MNEM_SHRD,
	MNEM_SHUFPD,
	MNEM_SHUFPS,
	MNEM_SIDT,
	MNEM_SLDT,
	MNEM_SMSW,
	MNEM_SQRTPD,
	MNEM_SQRTPS,
	MNEM_SQRTSD,
	MNEM_SQRTSS,
	MNEM_SS_,
	MNEM_STC,
	MNEM_STD,
	MNEM_STI,
	MNEM_STMXCSR,
	MNEM_STOSB,

	MNEM_STOSW_STOSD,
        MNEM_STOSW,
        MNEM_STOSD,

	MNEM_STR,
	MNEM_SUB,
	MNEM_SUBPD,
	MNEM_SUBPS,
	MNEM_SUBSD,
	MNEM_SUBSS,
	MNEM_SYSENTER,
	MNEM_SYSEXIT,
	MNEM_TEST,
	MNEM_UCOMISD,
	MNEM_UCOMISS,
	MNEM_UNARY_GRP_3_1A,
	MNEM_UNPCKHPD,
	MNEM_UNPCKHPS,
	MNEM_UNPCKLPD,
	MNEM_UNPCKLPS,
	MNEM_VERR,
	MNEM_VERW,
	MNEM_WAIT,
	MNEM_WBINVD,
	MNEM_WRMSR,
	MNEM_XADD,
	MNEM_XCHG,
	MNEM_XLAT,
	MNEM_XOR,
	MNEM_XORPD,
	MNEM_XORPS
};

static const char* Mnemonics[] =
      {"2-byte escape",
	"2-byte Illegal Opcodes UD2 1B",
	"aaa",
	"aad",
	"aam",
	"aas",
	"adc",
	"add",
	"addpd",
	"addps",
	"AddrSizePrefix",
	"addsd",
	"addss",
	"and",
	"andnpd",
	"andnps",
	"andpd",
	"andps",
	"arpl",
	"bound",
	"bsf",
	"bsr",
	"bswap",
	"bt",
	"btc",
	"btr",
	"bts",
	"call",
	"callf",
	"calln",

	"cbw/cwde",
        "cbw",
        "cwde",

	"clc",
	"cld",
	"clflush",
	"cli",
	"clts",
	"cmc",
	"cmova",
	"cmovae",
	"cmovb",
	"cmovbe",
	"cmove",
	"cmovl",
	"cmovle",
	"cmovne",
	"cmovge",
	"cmovg",
	"cmovnp",
	"cmovp",
	"cmovno",
	"cmovns",
	"cmovo",
	"cmovs",
	"cmp",
	"cmppd",
	"cmpps",
	"cmpsb",
	"cmpss",

	"cmpsw/cmpsd",
        "cmpsw",
        "cmpsd",

	"cmpxchg",
	"cmpxchg8b",
	"comisd",
	"comiss",
	"cpuid",
	"cs:",
	"cvtdq2pd",
	"cvtdq2ps",
	"cvtpd2dq",
	"cvtpd2pi",
	"cvtpd2ps",
	"cvtpi2pd",
	"cvtpi2ps",
	"cvtps2dq",
	"cvtps2pd",
	"cvtps2pi",
	"cvtsd2si",
	"cvtsd2ss",
	"cvtsi2sd",
	"cvtsi2ss",
	"cvtss2sd",
	"cvtss2si",
	"cvttpd2dq",
	"cvttpd2pi",
	"cvttps2dq",
	"cvttps2pi",
	"cvttsd2si",
	"cvttss2si",

	"cwd/cdq",
        "cwd",
        "cdq",

	"daa",
	"das",
	"dec",
	"div",
	"divpd",
	"divps",
	"divsd",
	"divss",
	"ds:",
	"emms",
	"enter",
	"Error",
	"es:",
	"ESC",
	"f2xm1",
	"fabs",
	"fadd",
	"faddp",
	"fbld",
	"fbstp",
	"fchs",
	"fclex",
	"fcmovb",
	"fcmovbe",
	"fcmove",
	"fcmovnb",
	"fcmovnbe",
	"fcmovne",
	"fcmovnu",
	"fcmovu",
	"fcom",
	"fcomi",
	"fcomip",
	"fcomp",
	"fcompp",
	"fcos",
	"fdecstp",
	"fdiv",
	"fdivp",
	"fdivr",
	"fdivrp",
	"ffree",
	"fiadd",
	"ficom",
	"ficomp",
	"fidiv",
	"fidivr",
	"fild",
	"fimul",
	"fincstp",
	"finit",
	"fist",
	"fistp",
	"fisub",
	"fisubr",
	"fld",
	"fld1",
	"fldcw",
	"fldenv",
	"fldl2e",
	"fldl2t",
	"fldlg2",
	"fldln2",
	"fldpi",
	"fldz",
	"fmul",
	"fmulp",
	"fnop",
	"fpatan",
	"fprem",
	"fprem1",
	"fptan",
	"frndint",
	"frstor",
	"fs:",
	"fsave",
	"fscale",
	"fsin",
	"fsincos",
	"fsqrt",
	"fst",
	"fstcw",
	"fstenv",
	"fstp",
	"fstsw",
	"fsub",
	"fsubp",
	"fsubr",
	"fsubrp",
	"ftst",
	"fucom",
	"fucomi",
	"fucomip",
	"fucomp",
	"fucompp",
	"fxam",
	"fxch",
	"fxrstor",
	"fxsave",
	"fxtract",
	"fyl2x",
	"fyl2xp1",
	"Grp 10 1A Invalid Opcode 1B",
	"Grp 11 1A - mov",
	"Grp 12 A1",
	"Grp 13 A1",
	"Grp 14 A1",
	"Grp 15 1A 1C",
	"Grp 6 1A",
	"Grp 7 1A",
	"Grp 8 1A",
	"Grp 9 1A",
	"gs:",
	"hlt",
	"idiv",
	"Immediate Grp 1 1A",
	"imul",
	"in",
	"inc",
	"inc/dec Grp 4 1A",
	"inc/dec Grp 5 1A",
	"insb",

	"insw/insd",
        "insw",
        "insd",

	"int",
	"into",
	"invd",
	"invlpg",
	"iret",
	"ja",
	"jae",
	"jb",
	"jbe",
	"je",
	"jl",
	"jle",
	"jne",
	"jge",
	"jg",
	"jnp",
	"jp",
	"ja",
	"jae",
	"jb",
	"jbe",

	"jcxz/jecxz",
        "jcxz",
        "jecxz",

	"je",
	"jg",
	"jge",
	"jl",
	"jle",
	"jmp far",
	"jmp near",
	"jmp short",
	"jmpf",
	"jmpn",
	"jne",
	"jno",
	"jnp",
	"jns",
	"jo",
	"jp",
	"js",
	"lahf",
	"lar",
	"ldmxcsr",
	"lds",
	"lea",
	"leave",
	"les",
	"lfence",
	"lfs",
	"lgdt",
	"lgs",
	"lidt",
	"lldt",
	"lmsw",
	"lock",
	"lodsb",

	"lodsw/lodsd",
        "lodsw",
        "lodsd",

	"loop",
	"loope",
	"loopne",
	"lsl",
	"lss",
	"ltr",
	"maskmovdqu",
	"maskmovq",
	"maxpd",
	"maxps",
	"maxsd",
	"maxss",
	"mfence",
	"minpd",
	"minps",
	"minsd",
	"minss",
	"mmx ud",
	"mov",
	"movapd",
	"movaps",
	"movd",
	"movdq2q",
	"movdqa",
	"movdqu",
	"movhpd",
	"movhps",
	"movlpd",
	"movlps",
	"movmskpd",
	"movmskps",
	"movntdq",
	"movnti",
	"movntpd",
	"movntps",
	"movntq",
	"movq",
	"movq2dq",
	"movsb",
	"movss",

	"movsw/movsd",
        "movsw",
        "movsd",

	"movsx",
	"movupd",
	"movups",
	"movzx",
	"mul",
	"mulpd",
	"mulps",
	"mulsd",
	"mulss",
	"neg",
	"nop",
	"not",
	"OpdSizePrefix",
	"or",
	"orpd",
	"orps",
	"out",
	"outsb",

	"outsw/outsd",
        "outsw",
        "outsd",

	"packssdw",
	"packsswb",
	"packuswb",
	"paddb",
	"paddd",
	"paddq",
	"paddsb",
	"paddsw",
	"paddusb",
	"paddusw",
	"paddw",
	"pand",
	"pandn",
	"pavgb",
	"pavgw",
	"pcmpeqb",
	"pcmpeqd",
	"pcmpeqw",
	"pcmpgtb",
	"pcmpgtd",
	"pcmpgtw",
	"pcmpqtd",
	"pextrw",
	"pinsrw",
	"pmaddwd",
	"pmaxsw",
	"pmaxub",
	"pminsw",
	"pminub",
	"pmovmskb",
	"pmulhuw",
	"pmulhw",
	"pmullw",
	"pmuludq",
	"pop",

	"popa/popad",
        "popa",
        "popad",

	"popf/popfd",
        "popf",
        "popfd",

	"por",
	"prefetch 1C {Grp 16 1A}",
	"prefetchnta",
	"prefetchT0",
	"prefetchT1",
	"prefetchT2",
	"psadbw",
	"pshufd",
	"pshufhw",
	"pshuflw",
	"pshufw",
	"pslld",
	"pslldq",
	"psllq",
	"psllw",
	"psrad",
	"psraw",
	"psrld",
	"psrldq",
	"psrlq",
	"psrlw",
	"psubb",
	"psubd",
	"psubq",
	"psubsb",
	"psubsw",
	"psubusb",
	"psubusw",
	"psubw",
	"punpckhbw",
	"punpckhdq",
	"punpckhqdq",
	"punpckhwd",
	"punpcklbw",
	"punpckldq",
	"punpcklqdq",
	"punpcklwd",
	"push",

	"pusha/pushad",
        "pusha",
        "pushad",

	"pushf/pushfd",
        "pushf",
        "pushfd",

	"pxor",
	"rcl",
	"rcpps",
	"rcpss",
	"rcr",
	"rdmsr",
	"rdpmc",
	"rdtsc",
	"rep/repe",
	"repne",
	"ret",
	"retf",
	"rol",
	"ror",
	"rsm",
	"rsqrtps",
	"rsqrtss",
	"sahf",
	"sar",
	"sbb",
	"scasb",

	"scasw/scasd",
        "scasw",
        "scasd",

	"seta",//"set a/nbe",
	"setae",//"set ae/nb/nc",
	"setb",
	"setbe",
	"sete",
	"setl",
	"setle",
	"setne",
	"setge",
	"setg",
	"setnp",
	"setp",
	"setno",
	"setns",
	"seto",
	"sets",
	"sgdt",
	"Shift Grp 2 1A",
	"shl",
	"shld",
	"shr",
	"shrd",
	"shufpd",
	"shufps",
	"sidt",
	"sldt",
	"smsw",
	"sqrtpd",
	"sqrtps",
	"sqrtsd",
	"sqrtss",
	"ss:",
	"stc",
	"std",
	"sti",
	"stmxcsr",
	"stosb",

	"stosw/stosd",
        "stosw",
        "stosd",

	"str",
	"sub",
	"subpd",
	"subps",
	"subsd",
	"subss",
	"sysenter",
	"sysexit",
	"test",
	"ucomisd",
	"ucomiss",
	"Unary Grp 3 1A",
	"unpckhpd",
	"unpckhps",
	"unpcklpd",
	"unpcklps",
	"verr",
	"verw",
	"wait",
	"wbinvd",
	"wrmsr",
	"xadd",
	"xchg",
	"xlat",
	"xor",
	"xorpd",
	"xorps"};

static void PtrNames(BYTE Size, char *Buffer)
{
#define MAX_PTR_NAME_LENGTH	10
	switch(Size)
      {
      case 8:
       strcpy(Buffer,"byte ptr");
      break;
      case 16:
       strcpy(Buffer,"word ptr");
      break;
      case 32:
       strcpy(Buffer,"dword ptr");
      break;
      case 64:
       strcpy(Buffer,"qword ptr");
      break;
      case 80:
       strcpy(Buffer,"tbyte ptr");
      break;
      case 128:
       strcpy(Buffer,"dqword ptr");
      break;
      default:
       Buffer[0] = '\0';
      }
}

//Must be sync. with TRegister struct
static const char* RegNames[] =
        {"NONE",
         "eax","ecx","ebx","edx","esi","edi","esp","ebp",
         "ax","cx","bx","dx","si","di","sp","bp",
         "ah","ch","bh","dh","al","cl","bl","dl",
         "cs","ds","es","fs","gs","ss",
         "cr0","cr2","cr3","cr4",
         "dr0","dr1","dr2","dr3","dr6","dr7",
         "st(0)","st(1)","st(2)","st(3)","st(4)","st(5)","st(6)","st(7)",
         "mm0","mm1","mm2","mm3","mm4","mm5","mm6","mm7",
         "xmm0","xmm1","xmm2","xmm3","xmm4","xmm5","xmm6","xmm7",
         "ERROR","eip"};

#ifdef __BUILDING_THE_DLL
#define __EXPORT_TYPE __export
#else
#define __EXPORT_TYPE __import
#endif

#ifdef __cplusplus
extern "C"
{
#endif
bool __EXPORT_TYPE DisAsm(DWORD EIP,struct TInstr Instr,TAddrMode AddrMode,
            TOperSize OperSize,TAsmCode* ResultAsmCode);
#ifdef __cplusplus
}
#endif

#endif



