/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*      Debugger Engine                                                       *
*      File: DisAsmView.h                                                    *
*      Copyright (c) 2004-2005 Fysyuk Victor, <RockDbgSupport@ua.fm>         *
*      ALL RIGHTS RESERVED.                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#ifndef DisAsmViewH
#define DisAsmViewH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>
#include <Controls.hpp>
#include <Windows.h>
#include "DisAsmEngine.h"
#include "ExportsList.h"
#include "Debugger.h"
//---------------------------------------------------------------------------

typedef void __fastcall (__closure *TBufferChangeEvent)(TObject* Sender,int First,int Last);

class PACKAGE TDisAsmTextAttributes: public TPersistent
{
private:
      TColor FForeground;
      TColor FBackground;
      TFontStyles FStyle;
      TNotifyEvent FOnChange;
      void __fastcall SetForeground(TColor value);
      void __fastcall SetBackground(TColor value);
      void __fastcall SetStyle(TFontStyles value);
public:
        TDisAsmTextAttributes();
        virtual void __fastcall Assign(TPersistent* Source);
        __property TNotifyEvent OnChange  = { read=FOnChange, write=FOnChange };
__published:
        __property TColor Foreground  = { read=FForeground, write=SetForeground };
        __property TColor Background  = { read=FBackground, write=SetBackground };
        __property TFontStyles Style  = { read=FStyle, write=SetStyle };
};

class PACKAGE TDisAsmLineAttributes: public TPersistent
{
private:
      TColor FForeground;
      TColor FBackground;
      TNotifyEvent FOnChange;
      void __fastcall SetForeground(TColor value);
      void __fastcall SetBackground(TColor value);
public:
        TDisAsmLineAttributes();
        virtual void __fastcall Assign(TPersistent* Source);
        __property TNotifyEvent OnChange  = { read=FOnChange, write=FOnChange };
__published:
        __property TColor Foreground  = { read=FForeground, write=SetForeground };
        __property TColor Background  = { read=FBackground, write=SetBackground };
};

enum TBufferItemType {itAsmCode, itString};

struct TBufferItem
{
        TBufferItemType type;
        TAsmCode AsmCode;
        DWORD Address;          //Only for AsmCode
        AnsiString String;
        TBufferItem* Prior;
        TBufferItem* Next;
};

class PACKAGE TDisAsmViewBuffer
{
private:
        TBufferItem* FItems;
        int FCount;
        TBufferChangeEvent FOnBufferChange;
        void __fastcall SetItemAsm(int Index, TAsmCode value);
        TAsmCode __fastcall GetItemAsm(int Index);
        void __fastcall SetItemStr(int Index, AnsiString value);
        AnsiString __fastcall GetItemStr(int Index);
        TBufferItemType __fastcall GetItemType(int Index);
        DWORD __fastcall GetItemAddress(int Index);
public:
        TDisAsmViewBuffer();
        void Insert(int Index, TAsmCode AsmCode, DWORD Address);
        void Insert(int Index, AnsiString S);
        void Add(TAsmCode AsmCode, DWORD Address) { Insert(Count, AsmCode, Address); }
        void Add(AnsiString S) { Insert(Count, S); }
        void Delete(int Index);
        void Clear();
        __property int Count  = { read=FCount };
        __property TBufferChangeEvent OnBufferChange  = { read=FOnBufferChange, write=FOnBufferChange };
        __property TAsmCode ItemsAsm[int Index]  = { read=GetItemAsm, write=SetItemAsm };
        __property AnsiString ItemsStr[int Index]  = { read=GetItemStr, write=SetItemStr };
        __property TBufferItemType ItemType[int Index]  = { read=GetItemType };
        __property DWORD ItemAddress[int Index]  = { read=GetItemAddress };
};

enum TTokenType
{
        tokAddress,
        tokMnemonic,
        tokNumber,
        tokRegister,
        tokPtrName,
        tokSpace,
        tokSymbol,
        tokString
};

struct TLineElement
{
	TTokenType ElementType;
	union
	{
		DWORD 	Address;
		UINT  	MnemIndex;
		int		Number;
		TRegister	Register;
		BYTE		PtrLength;
		char		Symbol;
		char*		String;
	};	
};

enum TDisAsmViewStateFlag
{
        stfDoNotChangeBGColor,
        stfDoNotChangeFGColor,
        stfDoNotInvalidateWhenAttrChanged,
        stfLinesChanging,
        stfPossibleGutterClick,
};

typedef Set<TDisAsmViewStateFlag,stfDoNotChangeBGColor,stfPossibleGutterClick> TStateFlags;

typedef void __fastcall (__closure *TGutterClickEvnt)(TObject *Sender, int X, int Y, int Line);

class TAddressStack;

enum TLinesDrawMode
{
      ldmDraw,
      ldmGetSelectedElements,
      ldmGetLineElement,
};

class PACKAGE TCustomDisAsmView : public Controls::TCustomControl
{
private:
      void __fastcall WMEraseBkgnd(Messages::TWMEraseBkgnd &Msg);
      void __fastcall WMGetDlgCode(Messages::TWMNoParams & Message);
      void __fastcall WMHScroll(Messages::TWMScroll & Msg);
      void __fastcall WMKillFocus(Messages::TWMSetFocus & Message);
      void __fastcall WMSetFocus(Messages::TWMSetFocus & Message);
      void __fastcall WMVScroll(Messages::TWMScroll &Msg);

      int FLine;
      int FElementStart;
      int FElementLength;
      TLineElement FLineElement;
      AnsiString FElementStr;
      bool FExistNeededElement;
      int FElmX;

      TScrollStyle FScrollBars;
      Graphics::TBitmap *FControlBitmap;
	TStateFlags FState;
      DWORD FEip;
      TDisAsmViewBuffer* FBuffer;
      DWORD FProcessID;
      DWORD FAddress;
      TExportsList* FExportsList;
      TDebugger* FDebugger;
      TAddressStack* FNavigator;
      int FGutterWidth;
      int FTextOffset;
      int FCharWidth;
      int FLeftChar;
      int FCharsInWindow;
      int FLinesInWindow;
      int FLineHeight;
      TRect FInvalidateRect;
      int FOperandsOffset;
      int FMnemonicOffset;
      int FDisAsmAreaWidth;
      Types::TPoint FCaretXY;
      TColor FSelectColor;
      Types::TPoint FSelBegin;
      Types::TPoint FSelEnd;
      TLinesDrawMode FLinesDrawMode;
      Classes::TStrings* FSelBuffer;
      AnsiString FSelBufferLine;
      int FMouseWheelAccumulator;

      TDisAsmTextAttributes* FAddressAttr;
      TDisAsmTextAttributes* FMnemAttr;
      TDisAsmTextAttributes* FNumberAttr;
      TDisAsmTextAttributes* FRegAttr;
      TDisAsmTextAttributes* FPtrNamesAttr;
      TDisAsmTextAttributes* FSpaceAttr;
      TDisAsmTextAttributes* FCurrentAttr;
      TDisAsmTextAttributes* FSymbolAttr;
      TDisAsmTextAttributes* FStringAttr;
      TDisAsmLineAttributes* FExecutionPointAttr;
      TDisAsmLineAttributes* FEnabledBreakAttr;
      TDisAsmLineAttributes* FDisabledBreakAttr;
      Graphics::TIcon* FExecutionIcon;
      Graphics::TIcon* FEnableBreakIcon;
      Graphics::TIcon* FDisabledBreakIcon;
      Graphics::TIcon* FExecAndEnableBreakIcon;
      Graphics::TIcon* FExecAndDisabledBreakIcon;

      TGutterClickEvnt FOnGutterClick;

      bool __fastcall IsExecutionIconStored();
      bool __fastcall IsEnableIconStored();
	bool __fastcall IsDisabledIconStored();
	bool __fastcall IsExecAndEnableIconStored();
	bool __fastcall IsExecAndDisabledStored();

      bool __fastcall DrawJumpTarget(int LineIndex, int &X, int Y,
            TAsmOperand AsmOperand, BYTE CodeLen,
            bool IsCurrentEIP, TLineElement &Token);
      void __fastcall AttrChangeHandler(TObject* Sender);
      void __fastcall SetDebugger(TDebugger* value);
      void __fastcall SetExportsList(TExportsList* value);
	void __fastcall SetScrollBars(TScrollStyle value);
      void __fastcall UpdateScrollBars();
      void __fastcall SetFont(TFont* value);
      TFont* __fastcall GetFont();
      void __fastcall SetBuffer(TDisAsmViewBuffer* value);
      void __fastcall SetGutterWidth(int value);
      void __fastcall SetLeftChar(int value);
      void __fastcall SetProcessID(DWORD value);
      void __fastcall UpdateTextAttributes();
      void __fastcall BufferChangeHandler(TObject* Sender,int First,int Last);
      void __fastcall SetAddressAttr(TDisAsmTextAttributes* value);
      void __fastcall SetMnemAttr(TDisAsmTextAttributes* value);
      void __fastcall SetNumberAttr(TDisAsmTextAttributes* value);
      void __fastcall SetRegAttr(TDisAsmTextAttributes* value);
      void __fastcall SetPtrNamesAttr(TDisAsmTextAttributes* value);
      void __fastcall SetSpaceAttr(TDisAsmTextAttributes* value);
      void __fastcall SetCurrentAttr(TDisAsmTextAttributes* value);
      void __fastcall SetSymbolAttr(TDisAsmTextAttributes* value);
      void __fastcall DrawToken(int LineIndex, int &X, int Y, TLineElement Token, int HexDigits = 0);
      void __fastcall DrawOperand(int LineIndex, int &X, int Y, TAsmOperand AsmOperand,
       bool IsFirstOperand,BYTE CodeLen,bool CALLN_or_JMPN = false,
       bool IsCurrentEIP = false);
      void __fastcall DrawLine(int LineIndex, int X, int Y);
      void __fastcall SizeOrFontChanged(bool bFont);
      int __fastcall GetLinesInWindow();
      void __fastcall PaintGutter(TRect rcClip, int FirstLine, int LastLine);
      void __fastcall SetOperandsOffset(int value);
      void __fastcall SetMnemonicOffset(int value);
      void __fastcall SetStringAttr(TDisAsmTextAttributes* value);
      void __fastcall SetExecutionPointAttr(TDisAsmLineAttributes* value);
      void __fastcall SetEnabledBreakAttr(TDisAsmLineAttributes* value);
      void __fastcall SetDisabledBreakAttr(TDisAsmLineAttributes* value);
      void __fastcall SetExecutionIcon(Graphics::TIcon* value);
      void __fastcall SetEnableBreakIcon(Graphics::TIcon* value);
      void __fastcall SetDisabledBreakIcon(Graphics::TIcon* value);
      void __fastcall SetExecAndEnableBreakIcon(Graphics::TIcon* value);
      void __fastcall SetExecAndDisabledBreakIcon(Graphics::TIcon* value);
      void __fastcall SetDisAsmAreaWidth(int value);
      int __fastcall GetCaretX() { return FCaretXY.x; };
      int __fastcall GetCaretY() { return FCaretXY.y; };
      void __fastcall SetCaretX(int value);
      void __fastcall SetCaretY(int value);
      void __fastcall SetCaretXY(Types::TPoint value);
      void __fastcall SetSelectColor(TColor value);
      void __fastcall SetSelBegin(Types::TPoint value);
      void __fastcall SetSelEnd(Types::TPoint value);
      void __fastcall ScrollCommandsBack(int InstrCount);
      void __fastcall ScrollCommandsForward(int InstrCount);
      void __fastcall ScrollOnePageForward();
      DWORD __fastcall DisassemblyBackward(DWORD dwAddress, int InstrCount,
                  int &RealInstrCount, bool ExactCount);
      DWORD __fastcall AddDisAsmMemoryToBuffer(DWORD dwAddress);
protected:
      virtual void __fastcall CreateParams(TCreateParams & Params);
      virtual void __fastcall Paint();
      virtual void __fastcall Loaded();
      virtual void __fastcall Notification(TComponent* AComponent, TOperation Operation);
      DYNAMIC void __fastcall Resize(void);
      DYNAMIC void __fastcall KeyDown(Word &Key, Classes::TShiftState Shift);
      DYNAMIC void __fastcall MouseDown(TMouseButton Button, Classes::TShiftState Shift, int X, int Y);
      DYNAMIC void __fastcall MouseUp(TMouseButton Button, Classes::TShiftState Shift, int X, int Y);
      DYNAMIC void __fastcall MouseMove(Classes::TShiftState Shift, int X, int Y);
      DYNAMIC void __fastcall MouseWheelHandler(Messages::TMessage &Message);
      BEGIN_MESSAGE_MAP
            VCL_MESSAGE_HANDLER(WM_ERASEBKGND, Messages::TWMEraseBkgnd, WMEraseBkgnd)
            VCL_MESSAGE_HANDLER(WM_VSCROLL, Messages::TWMScroll, WMVScroll)
            VCL_MESSAGE_HANDLER(WM_HSCROLL, Messages::TWMScroll, WMHScroll)
            VCL_MESSAGE_HANDLER(WM_SETFOCUS, Messages::TWMSetFocus, WMSetFocus)
            VCL_MESSAGE_HANDLER(WM_KILLFOCUS, Messages::TWMSetFocus, WMKillFocus)
            VCL_MESSAGE_HANDLER(WM_GETDLGCODE, Messages::TWMNoParams, WMGetDlgCode)
      END_MESSAGE_MAP(TCustomControl)

public:
      __fastcall virtual TCustomDisAsmView(Classes::TComponent* Owner);
      __fastcall virtual ~TCustomDisAsmView();
      __property Canvas;
      void UpdateBuffer(DWORD dwAddress, BYTE Count);
      void ShowMemory(DWORD dwAddress, DWORD EIP, bool UpdateBuf = false, bool ScrollToTop = false);
      void ShowMemory(DWORD dwAddress, bool UpdateBuf = false, bool ScrollToTop = false)
      {
            if(FDebugger == NULL) return;
            DWORD EIP = FDebugger->GetEIP(FDebugger->CurrentThreadId);
            ShowMemory(dwAddress, EIP, UpdateBuf, ScrollToTop);
      }
      void __fastcall ClearBuffer();
      void __fastcall InvalidateLines(int FirstLine, int LastLine);
      void __fastcall InvalidateLine(int Line) {InvalidateLines(Line,Line);};
      void __fastcall InvalidateGutterLines(int FirstLine, int LastLine);
      void __fastcall InvalidateGutterLine(int Line) {InvalidateGutterLines(Line,Line);};
      void __fastcall LinesChanging();
      void __fastcall LinesChanged();
      int __fastcall GetLineByAddress(DWORD Address);
      void __fastcall UpdateCaret();
      void __fastcall SetSelection(Types::TPoint Begin, Types::TPoint End);
      void __fastcall GetSelText(Classes::TStrings* Buffer);
      bool __fastcall GetElement(int X, int Y, int &Line, int &ElmStart,
            int &ElmLength, TLineElement &Element, AnsiString &ElementString);
      Types::TPoint __fastcall GetCaretXYPixels();
      __property TDisAsmViewBuffer* Buffer  = { read=FBuffer, write=SetBuffer };
      __property DWORD ProcessID  = { read=FProcessID, write=SetProcessID };
      __property DWORD Address  = { read=FAddress };
      __property TDisAsmTextAttributes* CurrentAttr  = { read=FCurrentAttr, write=SetCurrentAttr };
      __property int LinesInWindow  = { read=GetLinesInWindow };
      __property int CaretX  = { read=GetCaretX, write=SetCaretX };
      __property int CaretY  = { read=GetCaretY, write=SetCaretY };
      __property Types::TPoint CaretXY  = { read=FCaretXY, write=SetCaretXY };
      __property int LeftChar  = { read=FLeftChar, write=SetLeftChar };
      __property int DisAsmAreaWidth  = { read=FDisAsmAreaWidth, write=SetDisAsmAreaWidth };
      __property int LineHeight  = { read=FLineHeight };
      __property Types::TPoint SelBegin  = { read=FSelBegin, write=SetSelBegin };
      __property Types::TPoint SelEnd  = { read=FSelEnd, write=SetSelEnd };
      __property TAddressStack* Navigator  = { read=FNavigator };
      __property TLinesDrawMode LinesDrawMode  = { read=FLinesDrawMode, write=FLinesDrawMode };

      __property TDisAsmTextAttributes* AddressAttr  = { read=FAddressAttr, write=SetAddressAttr };
      __property TDisAsmTextAttributes* MnemAttr  = { read=FMnemAttr, write=SetMnemAttr };
      __property TDisAsmTextAttributes* NumberAttr  = { read=FNumberAttr, write=SetNumberAttr };
      __property TDisAsmTextAttributes* RegAttr  = { read=FRegAttr, write=SetRegAttr };
      __property TDisAsmTextAttributes* PtrNamesAttr  = { read=FPtrNamesAttr, write=SetPtrNamesAttr };
      __property TDisAsmTextAttributes* SpaceAttr  = { read=FSpaceAttr, write=SetSpaceAttr };
      __property TDisAsmTextAttributes* SymbolAttr  = { read=FSymbolAttr, write=SetSymbolAttr };
      __property TDisAsmTextAttributes* StringAttr  = { read=FStringAttr, write=SetStringAttr };
      __property TDisAsmLineAttributes* ExecutionPointAttr  = { read=FExecutionPointAttr, write=SetExecutionPointAttr };
      __property TDisAsmLineAttributes* EnabledBreakAttr  = { read=FEnabledBreakAttr, write=SetEnabledBreakAttr };
      __property TDisAsmLineAttributes* DisabledBreakAttr  = { read=FDisabledBreakAttr, write=SetDisabledBreakAttr };
      __property Graphics::TIcon* ExecutionIcon  = { read=FExecutionIcon, write=SetExecutionIcon, stored = IsExecutionIconStored};
      __property Graphics::TIcon* EnableBreakIcon  = { read=FEnableBreakIcon, write=SetEnableBreakIcon, stored = IsEnableIconStored };
      __property Graphics::TIcon* DisabledBreakIcon  = { read=FDisabledBreakIcon, write=SetDisabledBreakIcon, stored = IsDisabledIconStored };
      __property Graphics::TIcon* ExecAndEnableBreakIcon  = { read=FExecAndEnableBreakIcon, write=SetExecAndEnableBreakIcon, stored = IsExecAndEnableIconStored };
      __property Graphics::TIcon* ExecAndDisabledBreakIcon  = { read=FExecAndDisabledBreakIcon, write=SetExecAndDisabledBreakIcon, stored = IsExecAndDisabledStored };
      __property TDebugger* Debugger  = { read=FDebugger, write=SetDebugger };
      __property TExportsList* ExportsList  = { read=FExportsList, write=SetExportsList };
      __property int GutterWidth  = { read=FGutterWidth, write=SetGutterWidth };
      __property int CharsInWindow  = { read=FCharsInWindow };
      __property TFont* Font  = { read=GetFont, write=SetFont };
      __property TColor SelectColor  = { read=FSelectColor, write=SetSelectColor };
      __property int OperandsOffset  = { read=FOperandsOffset, write=SetOperandsOffset, default=25 };
      __property int MnemonicOffset  = { read=FMnemonicOffset, write=SetMnemonicOffset, default=15 };
      __property TScrollStyle ScrollBars  = { read=FScrollBars, write=SetScrollBars, default=ssBoth };

      __property TGutterClickEvnt OnGutterClick  = { read=FOnGutterClick, write=FOnGutterClick };
public:
      #pragma option push -w-inl
	inline __fastcall TCustomDisAsmView(HWND ParentWindow) : Controls::TCustomControl(ParentWindow) { }
	#pragma option pop
};
//---------------------------------------------------------------------------

class TDisAsmView : public TCustomDisAsmView
{
__published:
      /* inherited properties */
      __property Align = {default=0};
      __property Anchors = {default=3};
      __property Constraints;
      __property Color = {default = clWhite};
      __property Ctl3D;
      __property Cursor;
      __property ParentCtl3D;
      __property Enabled = {default=1};
      __property Font;
      __property Height;
      __property HelpContext;
      __property HelpKeyword;
      __property HelpType;
      __property Hint;
      __property Left;
      __property Name;
      __property ParentColor = {default=1};
      __property ParentFont = {default=1};
      __property ParentShowHint = {default=1};
      __property PopupMenu;
      __property ShowHint;
      __property TabOrder = {default=-1};
      __property TabStop = { default=1 };
      __property Tag = {default=0};
      __property Top;
      __property Visible = {default=1};
      __property Width;
      /* inherited events */
      __property OnClick;
      __property OnDblClick;
      __property OnDragDrop;
      __property OnDragOver;
      __property OnEndDock;
      __property OnStartDock;
      __property OnEndDrag;
      __property OnEnter;
      __property OnExit;
      __property OnKeyDown;
      __property OnKeyPress;
      __property OnKeyUp;
      __property OnMouseDown;
      __property OnMouseMove;
      __property OnMouseUp;
      __property OnStartDrag;
      /* TCustomDisAsmView properties */
      __property AddressAttr;
      __property CharsInWindow;
      __property Debugger;
      __property DisabledBreakAttr;
      __property DisabledBreakIcon;
      __property DisAsmAreaWidth;
      __property EnabledBreakAttr;
      __property EnableBreakIcon;
      __property ExecAndEnableBreakIcon;
      __property ExecAndDisabledBreakIcon;
      __property ExecutionIcon;
      __property ExecutionPointAttr;
      __property ExportsList;
      __property GutterWidth;
      __property MnemAttr;
      __property MnemonicOffset;
      __property NumberAttr;
      __property OperandsOffset;
      __property PtrNamesAttr;
      __property RegAttr;
      __property ScrollBars;
      __property SelectColor = { default = clSilver };
      __property SpaceAttr;
      __property StringAttr;
      __property SymbolAttr;
      /* TCustomDisAsmView events */
      __property OnGutterClick;
public:
      #pragma option push -w-inl
	inline __fastcall virtual TDisAsmView(Classes::TComponent* AOwner) : TCustomDisAsmView(AOwner) { }
	#pragma option pop
	#pragma option push -w-inl
	inline __fastcall virtual ~TDisAsmView(void) { }
	#pragma option pop
public:
	#pragma option push -w-inl
	inline __fastcall TDisAsmView(HWND ParentWindow) : TCustomDisAsmView(ParentWindow) { }
	#pragma option pop
};

class PACKAGE TAddressStack
{
private:
      TCustomDisAsmView* FDisAsmView;
      TList* FStack;
      int FStackTop;
      bool __fastcall GetCanForward();
      bool __fastcall GetCanBack();
      DWORD __fastcall GetAddresses(int Index);
      int __fastcall GetCount();
public:
      __fastcall TAddressStack(TCustomDisAsmView* DisAsmView);
      __fastcall ~TAddressStack();
      void __fastcall Clear();
      void __fastcall Goto(DWORD Address);
      bool __fastcall GoForward();
      bool __fastcall GoBack();
      __property bool CanForward  = { read=GetCanForward };
      __property bool CanBack  = { read=GetCanBack };
      __property DWORD Addresses[int Index]  = { read=GetAddresses };
      __property int Count  = { read=GetCount };
};

#endif

