/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*      Debugger Engine                                                       *
*      File: DumpView.h                                                      *
*      Copyright (c) 2004-2005 Fysyuk Victor, <RockDbgSupport@ua.fm>         *
*      ALL RIGHTS RESERVED.                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#ifndef DumpViewH
#define DumpViewH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>
#include <Controls.hpp>
#include <Grids.hpp>
#include "Debugger.h"
//---------------------------------------------------------------------------

#pragma option push -b-
enum TDumpDisplayFormat
{
      dfBytes,
      dfWords,
      dfDwords
};
#pragma option pop

struct TDataItem
{
      BYTE Data;
      bool IsValid;
      TDataItem *Next;
};

struct TCell
{
      int Row;
      int Col;
};

class TDumpViewData;

class PACKAGE TCustomDumpView : public TCustomDrawGrid
{
friend TDumpViewData;
private:
      TDumpViewData* FBytes;
      DWORD FAddress;
      DWORD FProcessID;
      TDebugger *FDebugger;
	TScrollStyle FScrollBars;
      int FBytesPerLineWhenBytesFormat;
      int FBytesPerLineWhenWordsFormat;
      int FBytesPerLineWhenDwordsFormat;
      bool FStackStyle;
      char __fastcall ByteToChar(BYTE Byte) const
      {return (Byte < 0x20 ? '.':(char)Byte);}
      void __fastcall SetProcessID(DWORD value);
	void __fastcall SetScrollBars(TScrollStyle value);
	void __fastcall UpdateScrollBars();
	virtual void __fastcall CreateParams(TCreateParams & Params);
	void __fastcall WMVScroll(Messages::TWMScroll & Msg);
      void __fastcall SetDisplayFormat(TDumpDisplayFormat value);
      void __fastcall SetStackStyle(bool value);
      void __fastcall SetDebugger(TDebugger* value);
protected:
	virtual void __fastcall DrawCell(int ACol, int ARow, const Types::TRect & ARect, TGridDrawState AState);
      void __fastcall Loaded();
      virtual void __fastcall Notification(TComponent* AComponent, TOperation Operation);
      DYNAMIC void __fastcall Resize(void);
      __property int BytesPerLineWhenBytesFormat  = { read=FBytesPerLineWhenBytesFormat, write=FBytesPerLineWhenBytesFormat };
	__property int BytesPerLineWhenWordsFormat  = { read=FBytesPerLineWhenWordsFormat, write=FBytesPerLineWhenWordsFormat };
	__property int BytesPerLineWhenDwordsFormat  = { read=FBytesPerLineWhenDwordsFormat, write=FBytesPerLineWhenDwordsFormat };
      TDumpDisplayFormat FDisplayFormat;
      int GetBytesPerLine();
      __property bool StackStyle  = { read=FStackStyle, write=SetStackStyle };
	BEGIN_MESSAGE_MAP
		VCL_MESSAGE_HANDLER(WM_VSCROLL, Messages::TWMScroll, WMVScroll)
	END_MESSAGE_MAP(TCustomDrawGrid)
public:
	__fastcall TCustomDumpView(TComponent* Owner);
      __fastcall ~TCustomDumpView();
      void __fastcall UpdateBuffer(DWORD dwAddress, BYTE Count);
      void __fastcall ShowMemory(DWORD dwAddress);
      TCell __fastcall AddressToColRow(DWORD dwAddress);
      DWORD __fastcall ColRowToAddress(TCell ColRow);
	bool __fastcall WriteToMemory(DWORD BaseAddress,DWORD *Values, DWORD Size);
      __property TDumpViewData* Bytes  = { read=FBytes };
      __property DWORD Address  = { read=FAddress };
      __property DWORD ProcessID  = { read=FProcessID, write=SetProcessID };
      __property TDebugger *Debugger  = { read=FDebugger, write=SetDebugger };
	__property TScrollStyle ScrollBars  = { read=FScrollBars, write=SetScrollBars, default = ssVertical };
      __property TDumpDisplayFormat DisplayFormat  = { read=FDisplayFormat, write=SetDisplayFormat, default=dfBytes };
};

class TDumpView : public TCustomDumpView
{
__published:
      /* inherited properties */
      __property Align;
      __property Anchors;
      __property Constraints;
      __property Color = {default = clBtnFace};
      __property Ctl3D;
      __property Cursor;
      __property ParentCtl3D;
      __property Enabled;
      __property Font;
      __property Height;
      __property HelpContext;
      __property HelpKeyword;
      __property HelpType;
      __property Hint;
      __property Left;
      __property Name;
      __property ParentColor;
      __property ParentFont;
      __property ParentShowHint;
      __property PopupMenu;
      __property ShowHint;
      __property TabOrder;
      __property TabStop = { default=true };
      __property Tag;
      __property Top;
      __property Visible;
      __property Width;
      /* inherited events */
      __property OnClick;
      __property OnDblClick;
      __property OnDragDrop;
      __property OnDragOver;
      __property OnEndDock;
      __property OnStartDock;
      __property OnEndDrag;
      __property OnEnter;
      __property OnExit;
      __property OnKeyDown;
      __property OnKeyPress;
      __property OnKeyUp;
      __property OnMouseDown;
      __property OnMouseMove;
      __property OnMouseUp;
      __property OnSelectCell;
      __property OnStartDrag;
      /* TCustomDumpView properties */
      __property Debugger;
	__property ScrollBars;
      __property DisplayFormat;
      __property StackStyle;
public:
      #pragma option push -w-inl
	inline __fastcall virtual TDumpView(Classes::TComponent* AOwner) : TCustomDumpView(AOwner) { }
	#pragma option pop
	#pragma option push -w-inl
	inline __fastcall virtual ~TDumpView(void) { }
	#pragma option pop
};

/*
*   DisplayFormat = dfBytes,  ColCount = 2 + 8 = 10.
*   DisplayFormat = dfWords,  ColCount = 2 + 4 = 6.
*   DisplayFormat = dfDwords,  ColCount = 2 + 2 = 4.
*/

class PACKAGE TDumpViewData : public TPersistent
{
private:
      TDataItem *FData;
      TCustomDumpView *FCustomDumpView;
      int FCount;
      bool FAutoInvalidate;
      BYTE __fastcall GetData(int Index);
      void __fastcall SetData(int Index, BYTE value);
      bool __fastcall GetIsValid(int Index);
public:
      TDumpViewData(TCustomDumpView *CustomDumpView);
      void __fastcall Clear();
      __fastcall ~TDumpViewData();
      void __fastcall Add(BYTE Value, bool IsValid = true);
      __property BYTE Data[int Index] = { read=GetData, write=SetData };
      __property bool IsValid[int Index] = { read=GetIsValid };
      __property int Count  = { read=FCount };
      __property bool AutoInvalidate  = { read=FAutoInvalidate, write=FAutoInvalidate };
};
//---------------------------------------------------------------------------
#endif
