/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*      Debugger Engine                                                       *
*      File: ExecBreakpoint.h                                                *
*      Copyright (c) 2004-2005 Fysyuk Victor, <RockDbgSupport@ua.fm>         *
*      ALL RIGHTS RESERVED.                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#ifndef ExecBreakpointH
#define ExecBreakpointH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>
#include <Contnrs.hpp>
//---------------------------------------------------------------------------

enum TExecBreakpointsListChangeType { ctAdd, ctDelete, ctClear, ctEdit,ctGroupedChange };
#define EI_CHANGED_ENABLED				0x01
#define EI_CHANGED_PASS_COUNT				0x02
#define EI_CHANGED_CURRENT_PASS_COUNT		0x04
#define EI_CHANGED_GROUP				0x08
#define EI_CHANGED_LOG_MESSAGE			0x10
#define EI_CHANGED_ENABLE_GROUP			0x20
#define EI_CHANGED_DISABLE_GROUP			0x40
#define EI_CHANGED_CONDITION				0x80
#define EI_CHANGED_EVAL_EXPRESSION			0x100

class TExecBreakpoint;

struct TExecBreakpointsListChangeInfo
{
	union
      {
		TExecBreakpoint *AddedBP;
      	TExecBreakpoint *EditedBP;
            union
            {
      	      DWORD DeletedBPAddress; // for CodeBP
                  struct                  // for SourceBP
                  {
                        DWORD Address;
                        DWORD Line;
                        char* File;
                  };
            };
      };
      WORD Flags;
};

typedef void __fastcall (__closure *TExecBreakpointChangeEvent)(TObject* Sender, WORD ChangeFlags);
typedef void __fastcall (__closure *TExecBreakpointsListChangeEvent)(TObject* Sender,
	TExecBreakpointsListChangeType ChangeType, TExecBreakpointsListChangeInfo ChangeInfo);

class TExecBreakpointsList;

class PACKAGE TExecBreakpoint : public TPersistent
{
friend TExecBreakpointsList;
private:
      DWORD FPassCount;
      DWORD FCurrentPassCount;
      AnsiString FGroup;
      AnsiString FLogMessage;
      AnsiString FEnableGroup;
      AnsiString FDisableGroup;
      TExecBreakpointChangeEvent FOnChange;
	AnsiString FCondition;
	AnsiString FEvalExpression;
      void __fastcall SetPassCount(DWORD value);
      void __fastcall SetCurrentPassCount(DWORD value);
      void __fastcall SetGroup(AnsiString value);
      void __fastcall SetLogMessage(AnsiString value);
      void __fastcall SetEnableGroup(AnsiString value);
      void __fastcall SetDisableGroup(AnsiString value);
	void __fastcall SetCondition(AnsiString value);
	void __fastcall SetEvalExpression(AnsiString value);
protected:
      bool FEnabled;
      virtual void __fastcall SetEnabled(bool value) = 0;
public:
      __fastcall TExecBreakpoint();
      __property bool Enabled  = { read=FEnabled, write=SetEnabled };
      __property DWORD PassCount  = { read=FPassCount, write=SetPassCount };
      __property DWORD CurrentPassCount  = { read=FCurrentPassCount, write=SetCurrentPassCount };
      __property AnsiString Group  = { read=FGroup, write=SetGroup };
      __property AnsiString LogMessage  = { read=FLogMessage, write=SetLogMessage };
      __property AnsiString EnableGroup  = { read=FEnableGroup, write=SetEnableGroup };
      __property AnsiString DisableGroup  = { read=FDisableGroup, write=SetDisableGroup };
      __property TExecBreakpointChangeEvent OnChange = { read=FOnChange, write=FOnChange };
	__property AnsiString Condition  = { read=FCondition, write=SetCondition };
	__property AnsiString EvalExpression  = { read=FEvalExpression, write=SetEvalExpression };
};

class PACKAGE TExecBreakpointsList : public TComponent
{
private:
      TObjectList *FBreakpoints;
      TExecBreakpointsListChangeEvent FOnChange;
      TExecBreakpoint* __fastcall GetBreakpoints(int Index);
      void __fastcall BPChangeHandler(TObject* Sender, WORD ChangeFlags);
      int __fastcall GetCount();
protected:
      bool FItemsChanging;
      void __fastcall Add(TExecBreakpoint* Breakpoint);
      void __fastcall Delete(int Index);
      __property TExecBreakpoint *Breakpoints[int Index] = { read=GetBreakpoints };
public:
      __fastcall TExecBreakpointsList(TComponent* Owner);
      __fastcall ~TExecBreakpointsList();
	void __fastcall Clear();
      void EnableGroup(AnsiString GroupName);
      void DisableGroup(AnsiString GroupName);
      void __fastcall BeginChanging();
      void __fastcall EndChanging(bool bNotifyAboutChanges = true);
      __property TExecBreakpointsListChangeEvent OnChange = { read=FOnChange, write=FOnChange };
      __property int Count  = { read=GetCount };
};

#endif
