/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
*      Debugger Engine                                                       *
*      File: ExportsList.h                                                   *
*      Copyright (c) 2004-2005 Fysyuk Victor, <RockDbgSupport@ua.fm>         *
*      ALL RIGHTS RESERVED.                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#ifndef ExportsListH
#define ExportsListH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Classes.hpp>
//---------------------------------------------------------------------------
struct TExprtFunc
{
      AnsiString Name;
      DWORD EntryPoint;
      DWORD OriginalEntryPoint;
      DWORD Ordinal;
};

class PACKAGE TExports
{
private:
      DWORD FFunctionsCount;
      DWORD FNamesCount;
      TExprtFunc *FFunctions;
      AnsiString FDllName;
      bool FIsCorrected;
      AnsiString FImageFileName;
      DWORD FModuleBaseAddress;
      DWORD FImageSize;
      TExprtFunc __fastcall GetFunctions(DWORD Index) const;
      void RestoreOriginalEntryPoints();
public:
      TExports();
      TExports(const TExports& Source);
      ~TExports();
      bool LoadExport(AnsiString FileName);
      void UnloadExports();
      void CorrectAddresses(DWORD ModuleBase);
      AnsiString __fastcall GetFuncNameByAddress(DWORD FuncAddress);
      DWORD __fastcall GetFunctionAddress(AnsiString Name);
	bool LoadExport(HANDLE hFile);
      __property DWORD FunctionsCount  = { read=FFunctionsCount };
      __property DWORD NamesCount  = { read=FNamesCount };
      __property AnsiString DllName  = { read=FDllName };
      __property TExprtFunc Functions[DWORD Index]  = { read=GetFunctions };
      __property bool IsCorrected  = { read=FIsCorrected };
      __property AnsiString ImageFileName  = { read=FImageFileName };
      __property DWORD ModuleBaseAddress  = { read=FModuleBaseAddress };
      __property DWORD ImageSize  = { read=FImageSize };
};

struct TExportsItem
{
      TExports *Exports;
      TExportsItem *NextItem;
};

class PACKAGE TExportsList : public TComponent
{
private:
      TExportsItem *FExportsItems;
      int FCount;
      TNotifyEvent FOnChange;
      TExports* __fastcall GetItems(int Index);
protected:
public:
      __fastcall TExportsList(TComponent* Owner);
      __fastcall ~TExportsList();
      TExports* AddExports(const TExports *Exports);
      TExports* AddExports(AnsiString FileName);
      TExports* AddExports(HANDLE hFile);
      void Clear();
      TExports* Find(AnsiString ImageName);
      void Delete(AnsiString ImageName);
      void Delete(DWORD BaseAddress);
      AnsiString __fastcall GetFuncNameByAddress(DWORD FuncAddress,bool ModuleName,bool RequiredName);
      DWORD __fastcall GetFunctionAddress(AnsiString FuncName,bool MustBeLoaded);
      AnsiString __fastcall GetModuleByAddress(DWORD Address);
      __property int Count  = { read=FCount };
      __property TExports* Items[int Index]  = { read=GetItems };
__published:
      __property TNotifyEvent OnChange  = { read=FOnChange, write=FOnChange };
};
//---------------------------------------------------------------------------
#endif
